from __future__ import annotations

from dataclasses import dataclass

from ..schemas.basic import File as SchemaFile


@dataclass(slots=True)
class File:
    id: str
    url: str
    filename: str | None = None
    mime: str | None = None

    @classmethod
    def from_schema(cls, f: SchemaFile) -> "File":
        return cls(id=f.id, url=f.url, filename=f.filename, mime=f.mime)



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from ..schemas.account import UserProfile as SchemaUserProfile
from ..schemas.enums import UserType

if TYPE_CHECKING:  # pragma: no cover
    from ..playerok import Playerok


@dataclass(slots=True)
class User:
    id: str
    username: str | None = None
    avatar_url: str | None = None
    role: UserType = UserType.USER
    is_online: bool | None = None
    is_blocked: bool | None = None
    rating: float | None = None
    reviews_count: int | None = None

    _client: "Playerok" = field(repr=False, default=None)

    @classmethod
    def _create(cls, *, client: "Playerok", id: str) -> "User":
        return cls(id=id, _client=client)

    def _merge_profile(self, profile: SchemaUserProfile) -> None:
        self.username = profile.username
        self.avatar_url = profile.avatar_url
        self.role = profile.role
        self.is_online = profile.is_online
        self.is_blocked = profile.is_blocked
        self.rating = profile.rating
        self.reviews_count = profile.reviews_count

    async def refresh(self) -> "User":
        await self._client._push_user(self.id)
        return self



import logging

from pydantic import BaseModel, ConfigDict, Field, model_validator

logger = logging.getLogger(__name__)


class ApiModel(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
    )


class UnimplementedApiModel(ApiModel):
    model_config = ConfigDict(
        extra="allow",
    )

    @model_validator(mode="before")
    @classmethod
    def _log_unimplemented(cls, data):
        if isinstance(data, dict):
            keys = list(data.keys())
            logger.warning(
                f"Model {cls.__name__} is unimplemented. Received keys: {keys}. "
                "Add these fields to the model if needed."
            )
        return data


class PageInfo(ApiModel):
    start_cursor: str | None = Field(None, alias="startCursor")
    end_cursor: str | None = Field(None, alias="endCursor")
    has_previous_page: bool | None = Field(None, alias="hasPreviousPage")
    has_next_page: bool | None = Field(None, alias="hasNextPage")


class File(ApiModel):
    id: str = Field(alias="id")
    url: str = Field(alias="url")
    filename: str | None = Field(None, alias="filename")
    mime: str | None = Field(None, alias="mime")



from .basic import ApiModel, File, PageInfo, UnimplementedApiModel  # noqa: I001
from .enums import UserType, QueryID  # noqa: I001

from .account import (
    Account,
    AccountBalance,
    AccountDealsStats,
    AccountIncomingDealsStats,
    AccountItemsStats,
    AccountOutgoingDealsStats,
    AccountProfile,
    AccountStats,
    UserProfile,
)

from .enums import (
    BankCardTypes,
    ChatMessageButtonTypes,
    ChatStatuses,
    ChatTypes,
    EventTypes,
    GameCategoryAgreementIconTypes,
    GameCategoryAutoConfirmPeriods,
    GameCategoryDataFieldInputTypes,
    GameCategoryDataFieldTypes,
    GameCategoryInstructionTypes,
    GameCategoryOptionTypes,
    GameType,
    ItemDealDirections,
    ItemDealStatuses,
    ItemLogEvents,
    ItemsSortOptions,
    ItemStatuses,
    PriorityTypes,
    ReviewStatuses,
    SortDirections,
    TransactionDirections,
    TransactionOperations,
    TransactionPaymentMethodIds,
    TransactionProviderDirections,
    TransactionProviderIds,
    TransactionStatuses,
)
from .games import (
    Game,
    GameCategory,
    GameCategoryAgreement,
    GameCategoryAgreementList,
    GameCategoryDataField,
    GameCategoryDataFieldList,
    GameCategoryInstruction,
    GameCategoryInstructionList,
    GameCategoryObtainingType,
    GameCategoryObtainingTypeList,
    GameCategoryOption,
    GameCategoryProps,
    GameList,
    GameProfile,
)
from .items import (
    Item,
    ItemList,
    ItemLog,
    ItemPriorityStatus,
    ItemPriorityStatusPriceRange,
    MyItem,
)
from .deals import (
    ItemDeal,
    ItemDealList,
    Review,
    ReviewList,
)
from .chats import (
    Chat,
    ChatList,
    ChatMessage,
    ChatMessageButton,
    ChatMessageList,
    Event,
    Moderator,
)
from .transactions import (
    SBPBankMember,
    Transaction,
    TransactionList,
    TransactionPaymentMethod,
    TransactionProvider,
    TransactionProviderLimitRange,
    TransactionProviderLimits,
    TransactionProviderProps,
    TransactionProviderRequiredUserData,
    UserBankCard,
    UserBankCardList,
)

__all__ = [
    "Account",
    "AccountBalance",
    "AccountDealsStats",
    "AccountIncomingDealsStats",
    "AccountItemsStats",
    "AccountOutgoingDealsStats",
    "AccountProfile",
    "AccountStats",
    "UserProfile",
    "QueryID",
    "GameType",
    "ApiModel",
    "UnimplementedApiModel",
    "PageInfo",
    "File",
    "UserType",
    "GameCategoryDataFieldTypes",
    "EventTypes",
    "ItemLogEvents",
    "TransactionOperations",
    "TransactionDirections",
    "TransactionStatuses",
    "TransactionPaymentMethodIds",
    "TransactionProviderDirections",
    "TransactionProviderIds",
    "BankCardTypes",
    "ItemDealStatuses",
    "ItemDealDirections",
    "ChatTypes",
    "ChatStatuses",
    "ChatMessageButtonTypes",
    "ItemStatuses",
    "ReviewStatuses",
    "SortDirections",
    "PriorityTypes",
    "GameCategoryAgreementIconTypes",
    "GameCategoryOptionTypes",
    "GameCategoryDataFieldInputTypes",
    "GameCategoryAutoConfirmPeriods",
    "GameCategoryInstructionTypes",
    "Game",
    "GameCategory",
    "GameCategoryAgreement",
    "GameCategoryAgreementList",
    "GameCategoryDataField",
    "GameCategoryDataFieldList",
    "GameCategoryInstruction",
    "GameCategoryInstructionList",
    "GameCategoryObtainingType",
    "GameCategoryObtainingTypeList",
    "GameCategoryOption",
    "GameCategoryProps",
    "GameList",
    "GameProfile",
    "Item",
    "ItemList",
    "ItemLog",
    "ItemPriorityStatus",
    "ItemPriorityStatusPriceRange",
    "MyItem",
    "ItemDeal",
    "ItemDealList",
    "Review",
    "ReviewList",
    "Chat",
    "ChatList",
    "ChatMessage",
    "ChatMessageButton",
    "ChatMessageList",
    "Event",
    "Moderator",
    "SBPBankMember",
    "Transaction",
    "TransactionList",
    "TransactionPaymentMethod",
    "TransactionProvider",
    "TransactionProviderLimitRange",
    "TransactionProviderLimits",
    "TransactionProviderProps",
    "TransactionProviderRequiredUserData",
    "UserBankCard",
    "UserBankCardList",
    "ItemsSortOptions",
]



from __future__ import annotations

from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import Game, GameCategoryInstructionTypes, GameList, GameType
from ..schemas.games import (
    GameCategory,
    GameCategoryAgreementList,
    GameCategoryDataFieldList,
    GameCategoryDataFieldTypes,
    GameCategoryInstructionList,
    GameCategoryObtainingTypeList,
)
from ..transport import PlayerokTransport


class RawGamesService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_games(
        self, count: int = 24, type: GameType | None = None, cursor: str | None = None
    ) -> GameList | None:
        response = await self._transport.request(
            "post", "graphql", GQL.get_games(count=count, type=type, cursor=cursor)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "games"))
        if data is None:
            return None
        return GameList(**data)

    async def get_game(self, id: str | None = None, slug: str | None = None) -> Game | None:
        if id is None and slug is None:
            raise ValueError("Can't get game without id or slug")

        response = await self._transport.request("post", "graphql", GQL.get_game(id=id, slug=slug))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "game"))
        if data is None:
            return None
        return Game(**data)

    async def get_game_category(
        self, game_id: str | None = None, slug: str | None = None, id: str | None = None
    ) -> GameCategory | None:
        if id is None and slug is None and game_id is None:
            raise ValueError("Can't get game category without id or slug or game_id")

        response = await self._transport.request(
            "post", "graphql", GQL.get_game_category(game_id=game_id, slug=slug, id=id)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "gameCategory"))
        if data is None:
            return None
        return GameCategory(**data)

    async def get_game_category_agreements(
        self,
        game_category_id: str,
        user_id: str,
        count: int = 24,
        cursor: str | None = None,
    ) -> GameCategoryAgreementList | None:
        if not game_category_id:
            raise ValueError("game_category_id is required")

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_game_category_agreements(
                game_category_id=game_category_id,
                user_id=user_id,
                count=count,
                cursor=cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "gameCategoryAgreements"))
        if data is None:
            return None
        return GameCategoryAgreementList(**data)

    async def get_game_category_obtaining_types(
        self,
        game_category_id: str,
        count: int = 24,
        cursor: str | None = None,
    ) -> GameCategoryObtainingTypeList | None:
        if not game_category_id:
            raise ValueError("game_category_id is required")

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_game_category_obtaining_types(
                game_category_id=game_category_id,
                count=count,
                cursor=cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "gameCategoryObtainingTypes"))
        if data is None:
            return None
        return GameCategoryObtainingTypeList(**data)

    async def get_game_category_instructions(
        self,
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryInstructionTypes | None = None,
        cursor: str | None = None,
    ) -> GameCategoryInstructionList | None:
        if not game_category_id:
            raise ValueError("game_category_id is required")
        if not obtaining_type_id:
            raise ValueError("obtaining_type_id is required")

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_game_category_instructions(
                game_category_id=game_category_id,
                obtaining_type_id=obtaining_type_id,
                count=count,
                type=type,
                cursor=cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "gameCategoryInstructions"))
        if data is None:
            return None
        return GameCategoryInstructionList(**data)

    async def get_game_category_data_fields(
        self,
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryDataFieldTypes | None = None,
        cursor: str | None = None,
    ) -> GameCategoryDataFieldList | None:
        if not game_category_id:
            raise ValueError("game_category_id is required")
        if not obtaining_type_id:
            raise ValueError("obtaining_type_id is required")

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_game_category_data_fields(
                game_category_id=game_category_id,
                obtaining_type_id=obtaining_type_id,
                count=count,
                type=type,
                cursor=cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "gameCategoryDataFields"))
        if data is None:
            return None
        return GameCategoryDataFieldList(**data)



from __future__ import annotations

from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import (
    Item,
    ItemList,
    ItemPriorityStatus,
    ItemsSortOptions,
    MyItem,
    TransactionPaymentMethodIds,
    TransactionProviderIds,
)
from ..transport import PlayerokTransport


class RawItemsService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_items(
        self,
        count: int = 24,
        cursor: str | None = None,
        game_id: str | None = None,
        category_id: str | None = None,
        minimal_price: int | None = None,
        maximal_price: int | None = None,
        has_discount: bool | None = None,
        has_reviews: bool | None = None,
        attributes: list[dict[str, str]] | None = None,
        search: str | None = None,
        sort: ItemsSortOptions | None = None,
    ) -> ItemList | None:
        if game_id is None or category_id is None:
            raise ValueError("Can't get items without game_id and category_id")

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_items(
                count=count,
                cursor=cursor,
                game_id=game_id,
                category_id=category_id,
                minimal_price=minimal_price,
                maximal_price=maximal_price,
                has_discount=has_discount,
                has_reviews=has_reviews,
                attributes=attributes,
                search=search,
                sort=sort,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "items"))
        if data is None:
            return None
        return ItemList(**data)

    async def get_item(self, id: str | None = None, slug: str | None = None) -> Item | None:
        if id is None and slug is None:
            raise ValueError("Can't get item without id or slug")

        response = await self._transport.request("post", "graphql", GQL.get_item(id=id, slug=slug))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "item"))
        if data is None:
            return None
        return Item(**data)

    async def create_item(
        self,
        game_category_id: str,
        obtaining_type_id: str,
        name: str,
        price: int,
        description: str,
        options: dict[str, str],
        data_fields: dict[str, str],
        attachments: list[str],
    ) -> MyItem | None:
        payload_data_fields = [{"fieldId": k, "value": v} for k, v in data_fields.items()]

        payload = GQL.create_item(
            game_category_id=game_category_id,
            obtaining_type_id=obtaining_type_id,
            name=name,
            price=int(price),
            description=description,
            attributes=options,
            data_fields=payload_data_fields,
            attachments_count=len(attachments),
        )

        files = {str(i + 1): open(attachments[i], "rb") for i in range(len(attachments))}
        try:
            response = await self._transport.request("post", "graphql", payload, files=files)
        finally:
            for f in files.values():
                f.close()

        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "createItem"))
        if data is None:
            return None
        return MyItem(**data)

    async def update_item(
        self,
        id: str,
        name: str | None = None,
        price: int | None = None,
        description: str | None = None,
        options: dict[str, str] | None = None,
        data_fields: dict[str, str] | None = None,
        remove_attachments: list[str] | None = None,
        add_attachments: list[str] | None = None,
    ) -> MyItem | None:
        payload_data_fields = (
            [{"fieldId": k, "value": v} for k, v in data_fields.items()]
            if data_fields is not None
            else None
        )

        payload = GQL.update_item(
            id=id,
            name=name,
            price=int(price) if price is not None else None,
            description=description,
            attributes=options,
            data_fields=payload_data_fields,
            removed_attachments=remove_attachments,
            attachments_count=len(add_attachments) if add_attachments else 0,
        )

        files = (
            {str(i + 1): open(add_attachments[i], "rb") for i in range(len(add_attachments))}
            if add_attachments
            else None
        )

        try:
            response = await self._transport.request("post", "graphql", payload, files=files)
        finally:
            if files:
                for f in files.values():
                    f.close()

        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "updateItem"))
        if data is None:
            return None
        return MyItem(**data)

    async def remove_item(self, id: str) -> bool:
        response = await self._transport.request("post", "graphql", GQL.remove_item(id=id))
        raw = response.json()
        _raise_on_gql_errors(raw)
        return True

    async def publish_item(
        self,
        item_id: str,
        priority_status_id: str,
        transaction_provider_id: TransactionProviderIds = TransactionProviderIds.LOCAL,
    ) -> MyItem | None:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.publish_item(
                item_id=item_id,
                priority_status_id=priority_status_id,
                transaction_provider_id=transaction_provider_id.name,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "publishItem"))
        if data is None:
            return None
        return MyItem(**data)

    async def get_item_priority_statuses(
        self, item_id: str, price: int
    ) -> list[ItemPriorityStatus]:
        response = await self._transport.request(
            "post", "graphql", GQL.get_item_priority_statuses(item_id=item_id, price=price)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "itemPriorityStatuses")) or []
        return [ItemPriorityStatus(**status) for status in data]

    async def increase_item_priority_status(
        self,
        item_id: str,
        priority_status_id: str,
        payment_method_id: TransactionPaymentMethodIds | None = None,
        transaction_provider_id: TransactionProviderIds = TransactionProviderIds.LOCAL,
    ) -> MyItem | None:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.increase_item_priority_status(
                item_id=item_id,
                priority_status_id=priority_status_id,
                payment_method_id=payment_method_id.name if payment_method_id else None,
                transaction_provider_id=transaction_provider_id.name,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "increaseItemPriorityStatus"))
        if data is None:
            return None
        return MyItem(**data)



BASE_HEADERS = {
    "accept": "*/*",
    "accept-language": "en-US,en;q=0.9",
    "access-control-allow-headers": "sentry-trace, baggage",
    "apollo-require-preflight": "true",
    "apollographql-client-name": "web",
    "content-type": "application/json",
    "origin": "https://playerok.com",
    "priority": "u=1, i",
    "referer": "https://playerok.com/",
    "sec-ch-ua": '"Google Chrome";v="143", "Chromium";v="143", "Not A(Brand";v="24"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"macOS"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-origin",
    "x-gql-op": "viewer",
    "x-gql-path": "/products/[slug]",
    "x-timezone-offset": "-180",
}

EXAMPLE_USER_AGENTS = [
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 7_0_2; en-US) Gecko/20100101 Firefox/57.1",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 9_8_3) AppleWebKit/534.47 (KHTML, like Gecko) Chrome/51.0.1124.252 Safari/535",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_2_9) Gecko/20100101 Firefox/65.6",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_0_2) AppleWebKit/536.44 (KHTML, like Gecko) Chrome/49.0.2950.348 Safari/602",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 9_9_9; en-US) Gecko/20130401 Firefox/59.3",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_1_1; en-US) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/53.0.2619.208 Safari/535",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_8) Gecko/20130401 Firefox/59.9",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0) Gecko/20130401 Firefox/68.4",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_3_5; en-US) Gecko/20100101 Firefox/53.9",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 7_1_1; en-US) Gecko/20130401 Firefox/61.0",
]

CLOUDFLARE_SIGNATURES = [
    "<title>Just a moment...</title>",
    "window._cf_chl_opt",
    "Enable JavaScript and cookies to continue",
    "Checking your browser before accessing",
    "cf-browser-verification",
    "Cloudflare Ray ID",
]



from __future__ import annotations

from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import Chat, ChatList, ChatMessage, ChatMessageList, ChatStatuses, ChatTypes
from ..transport import PlayerokTransport


class RawChatService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_chats(
        self,
        user_id: str | None = None,
        count: int = 24,
        type: ChatTypes | None = None,
        status: ChatStatuses | None = None,
        cursor: str | None = None,
    ) -> ChatList | None:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_chats(user_id=user_id, count=count, type=type, status=status, cursor=cursor),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "chats"))
        if data is None:
            return None
        return ChatList(**data)

    async def get_chat(self, chat_id: str) -> Chat | None:
        response = await self._transport.request("post", "graphql", GQL.get_chat(chat_id=chat_id))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "chat"))
        if data is None:
            return None
        return Chat(**data)

    async def mark_chat_as_read(self, chat_id: str) -> Chat | None:
        response = await self._transport.request(
            "post", "graphql", GQL.mark_chat_as_read(chat_id=chat_id)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "markChatAsRead"))
        if data is None:
            return None
        return Chat(**data)

    async def get_chat_messages(
        self, chat_id: str, count: int = 24, after_cursor: str | None = None
    ) -> ChatMessageList | None:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_chat_messages(chat_id=chat_id, count=count, after_cursor=after_cursor),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "chatMessages"))
        if data is None:
            return None
        return ChatMessageList(**data)

    async def send_message(
        self,
        chat_id: str,
        text: str | None = None,
        photo_path: str | None = None,
        mark_as_read: bool = False,
    ) -> ChatMessage | None:
        if not text and not photo_path:
            raise ValueError("Either 'text' or 'photo_path' must be provided.")

        if mark_as_read:
            await self.mark_chat_as_read(chat_id=chat_id)
        if text and photo_path:
            await self.send_message(chat_id=chat_id, photo_path=photo_path)
            return await self.send_message(chat_id=chat_id, text=text)  # can't send both at once

        if photo_path:
            payload = GQL.create_chat_message_with_photo(chat_id=chat_id, text=text)
            with open(photo_path, "rb") as f:
                files = {"1": f}
                response = await self._transport.request("post", "graphql", payload, files=files)
        else:
            payload = GQL.create_chat_message(chat_id=chat_id, text=text)
            response = await self._transport.request("post", "graphql", payload)

        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "createChatMessage"))
        if data is None:
            return None
        return ChatMessage(**data)



import os
from typing import Any, Literal

from ..core.config import PlayerokConfig
from ..core.constants import CLOUDFLARE_SIGNATURES
from ..core.exceptions import CloudflareDetected


class PlayerokTransport:
    """
    Shared network transport for a single Playerok session.

    Owns one underlying TLS/HTTP client (tls_requests.AsyncClient) that is reused by all raw services.
    """

    def __init__(self, access_token: str | None = None, config: PlayerokConfig | None = None):
        # Lazy import so `aiosellers.playerok` can be imported without optional deps installed
        # (useful for tooling/tests in environments without network access).
        from tls_requests import AsyncClient  # type: ignore

        self._access_token = access_token or os.getenv("PLAYEROK_ACCESS_TOKEN")
        self._config = config or PlayerokConfig()

        if not self._access_token:
            raise RuntimeError(
                "Please provide playerok access token or fill PLAYEROK_ACCESS_TOKEN environment variable."
            )

        self._client = AsyncClient(
            cookies={"token": self._access_token},
            headers=self._config.headers,
            timeout=self._config.request_timeout,
        )

    @property
    def config(self) -> PlayerokConfig:
        return self._config

    @staticmethod
    def _raise_if_cloudflare(response: Any) -> None:
        # tls_requests.Response has .text property
        if any(sig in response.text for sig in CLOUDFLARE_SIGNATURES):
            raise CloudflareDetected("The cloudflare protection is detected.")

    async def request(
        self,
        method: Literal["get", "post"],
        url: str,
        payload: dict[str, Any] | None = None,
        *,
        headers: dict[str, str] | None = None,
        files: dict[str, Any] | None = None,
    ) -> Any:
        request_headers = self._config.headers.copy()
        if headers is not None:
            request_headers.update(headers)

        if not url.startswith("http"):
            url = self._config.base_url + url

        payload = payload or {}

        if method == "get":
            response = await self._client.get(url=url, headers=request_headers, params=payload)
        elif method == "post":
            if files:
                # Let the client set proper multipart boundary.
                request_headers.pop("content-type", None)
                response = await self._client.post(
                    url=url, headers=request_headers, data=payload, files=files
                )
            else:
                response = await self._client.post(url=url, headers=request_headers, json=payload)
        else:
            raise RuntimeError(f"Unsupported HTTP method: {method}")

        self._raise_if_cloudflare(response)
        return response

    async def close(self) -> None:
        await self._client.aclose()



from __future__ import annotations

import typing
from datetime import datetime

from pydantic import Field, model_validator

from . import (
    ApiModel,
    ChatMessageButtonTypes,
    ChatStatuses,
    ChatTypes,
    Game,
    Item,
    PageInfo,
)
from .account import UserProfile
from .basic import File, UnimplementedApiModel

if typing.TYPE_CHECKING:
    from .deals import ItemDeal
    from .transactions import Transaction


class Event(UnimplementedApiModel):
    """# TODO: Implement Event model fields."""

    pass


class Moderator(UnimplementedApiModel):
    """# TODO: Implement Moderator model fields."""

    pass


class ChatMessageButton(ApiModel):
    type: ChatMessageButtonTypes = Field(..., alias="type")
    url: str | None = Field(None, alias="url")
    text: str | None = Field(None, alias="text")


class ChatMessage(ApiModel):
    id: str = Field(..., alias="id")
    text: str | None = Field(None, alias="text")
    created_at: datetime = Field(..., alias="createdAt")
    deleted_at: datetime | None = Field(None, alias="deletedAt")
    is_read: bool = Field(..., alias="isRead")
    is_suspicious: bool | None = Field(None, alias="isSuspicious")
    is_bulk_messaging: bool | None = Field(None, alias="isBulkMessaging")
    file: File | None = Field(None, alias="file")
    game: Game | None = Field(None, alias="game")
    user: UserProfile | None = Field(None, alias="user")
    deal: ItemDeal | None = Field(None, alias="deal")
    item: Item | None = Field(None, alias="item")
    transaction: Transaction | None = Field(None, alias="transaction")
    moderator: Moderator | None = Field(None, alias="moderator")
    event_by_user: UserProfile | None = Field(None, alias="eventByUser")
    event_to_user: UserProfile | None = Field(None, alias="eventToUser")
    is_auto_response: bool | None = Field(None, alias="isAutoResponse")
    event: Event | str | None = Field(None, alias="event")
    buttons: list[ChatMessageButton] | None = Field(None, alias="buttons")


class ChatMessageList(ApiModel):
    messages: list[ChatMessage] = Field(..., alias="messages")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "messages" not in data:
            edges = data.get("edges") or []
            data["messages"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data


class Chat(ApiModel):
    id: str = Field(..., alias="id")
    type: ChatTypes = Field(ChatTypes.PM, alias="type")
    status: ChatStatuses | None = Field(None, alias="status")
    unread_messages_counter: int | None = Field(None, alias="unreadMessagesCounter")
    bookmarked: bool | None = Field(None, alias="bookmarked")
    is_texting_allowed: bool | None = Field(None, alias="isTextingAllowed")
    owner: UserProfile | None = Field(None, alias="owner")
    deals: list[ItemDeal] | None = Field(None, alias="deals")
    last_message: ChatMessage | None = Field(None, alias="lastMessage")
    users: list[UserProfile] | None = Field(None, alias="participants")
    started_at: datetime | None = Field(None, alias="startedAt")
    finished_at: datetime | None = Field(None, alias="finishedAt")


class ChatList(ApiModel):
    chats: list[Chat] = Field(..., alias="chats")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "chats" not in data:
            edges = data.get("edges") or []
            data["chats"] = [(edge or {}).get("node") for edge in edges if (edge or {}).get("node")]
        return data


# Resolve forward references after all models are defined
from .deals import ItemDeal  # noqa: E402
from .transactions import Transaction  # noqa: E402

ChatMessage.model_rebuild()
Chat.model_rebuild()
ChatList.model_rebuild()
ChatMessageList.model_rebuild()



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, AsyncIterator

from .cache import AsyncTTLCache
from .entities import Chat, Deal, User
from .raw import RawAPI
from .schemas import Account
from .schemas.account import AccountProfile
from .schemas.account import UserProfile as SchemaUserProfile
from .schemas.chats import Chat as SchemaChat
from .schemas.deals import ItemDeal as SchemaDeal
from .schemas.enums import ChatStatuses, ChatTypes, ItemDealDirections, ItemDealStatuses
from .transport import PlayerokTransport


@dataclass(slots=True)
class _Caches:
    users: AsyncTTLCache[str, Any]
    chats: AsyncTTLCache[str, Any]
    deals: AsyncTTLCache[str, Any]
    items: AsyncTTLCache[str, Any]
    transactions: AsyncTTLCache[str, Any]
    files: AsyncTTLCache[str, Any]


class Playerok:
    """
    High-level client facade.

    This class owns:
    - one transport (shared TLS client)
    - raw low-level API (`self.raw`)
    - caches (identity map + TTL)
    """

    def __init__(self, access_token: str | None = None):
        self._access_token = access_token
        self.transport: PlayerokTransport | None = None
        self.raw: RawAPI | None = None
        self.me: Account | None = None
        self.profile: AccountProfile | None = None

        ttl = 300.0
        self.cache = _Caches(
            users=AsyncTTLCache(ttl),
            chats=AsyncTTLCache(ttl),
            deals=AsyncTTLCache(ttl),
            items=AsyncTTLCache(ttl),
            transactions=AsyncTTLCache(ttl),
            files=AsyncTTLCache(ttl),
        )

    async def __aenter__(self) -> "Playerok":
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:  # noqa: ANN001
        await self.close()

    async def start(self) -> None:
        if self.transport is not None:
            return
        self.transport = PlayerokTransport(access_token=self._access_token)
        self.raw = RawAPI(self.transport)
        self.me = await self.raw.account.get_me()
        self.profile = await self.raw.account.get_account(self.me.username)

        # identity-map: create "me" as User entity too (optional convenience)
        self._get_user_identity(self.me.id)._merge_profile(
            SchemaUserProfile.model_validate(
                {"id": self.me.id, "username": self.me.username, "role": self.me.role}
            )
        )

    async def close(self) -> None:
        if self.transport is None:
            return
        await self.transport.close()
        self.transport = None
        self.raw = None

    # convenience proxies (me.*)
    @property
    def id(self) -> str | None:
        return self.me.id if self.me else None

    @property
    def username(self) -> str | None:
        return self.me.username if self.me else None

    @property
    def email(self) -> str | None:
        return self.me.email if self.me else None

    @property
    def balance(self) -> float | None:
        return self.profile.balance if self.profile else None

    # --------------------
    # identity-map helpers
    # --------------------
    def _get_user_identity(self, user_id: str) -> User:
        return self.cache.users.get_or_create(
            user_id, lambda: User._create(client=self, id=user_id)
        )

    def _get_chat_identity(self, chat_id: str) -> Chat:
        return self.cache.chats.get_or_create(
            chat_id, lambda: Chat._create(client=self, id=chat_id)
        )

    def _get_deal_identity(self, deal_id: str) -> Deal:
        return self.cache.deals.get_or_create(
            deal_id, lambda: Deal._create(client=self, id=deal_id)
        )

    def _push_user_profile(self, profile: SchemaUserProfile) -> User:
        user = self._get_user_identity(profile.id)
        user._merge_profile(profile)
        self.cache.users.touch(profile.id)
        return user

    def _push_chat_schema(self, chat: SchemaChat) -> Chat:
        entity = self._get_chat_identity(chat.id)
        entity._merge_schema(chat)
        self.cache.chats.touch(chat.id)
        return entity

    def _push_deal_schema(self, deal: SchemaDeal) -> Deal:
        entity = self._get_deal_identity(deal.id)
        entity._merge_schema(deal)
        self.cache.deals.touch(deal.id)
        return entity

    # --------------------
    # high-level get
    # --------------------
    async def get_user(self, user_id: str) -> User:
        async def fetch() -> User:
            profile = await self.raw.account.get_user(id=user_id)
            if profile is not None:
                return self._push_user_profile(profile)
            return self._get_user_identity(user_id)

        return await self.cache.users.get_or_fetch(user_id, fetch)

    async def _push_user(self, user_id: str) -> User:
        profile = await self.raw.account.get_user(id=user_id)
        if profile is not None:
            return self._push_user_profile(profile)
        return self._get_user_identity(user_id)

    async def get_deal(self, deal_id: str) -> Deal | None:
        async def fetch() -> Deal | None:
            deal = await self.raw.deals.get_deal(deal_id)
            if deal is None:
                return None
            return self._push_deal_schema(deal)

        return await self.cache.deals.get_or_fetch(deal_id, fetch)

    async def _push_deal(self, deal_id: str) -> Deal | None:
        deal = await self.raw.deals.get_deal(deal_id)
        if deal is None:
            return None
        return self._push_deal_schema(deal)

    async def get_chat(self, chat_id: str) -> Chat | None:
        async def fetch() -> Chat | None:
            chat = await self.raw.chats.get_chat(chat_id)
            if chat is None:
                return None
            return self._push_chat_schema(chat)

        return await self.cache.chats.get_or_fetch(chat_id, fetch)

    async def _push_chat(self, deal_id: str) -> Chat | None:
        deal = await self.raw.chats.get_chat(deal_id)
        if deal is None:
            return None
        return self._push_chat_schema(deal)

    # --------------------
    # iterators / queries
    # --------------------
    async def iter_chats(
        self,
        *,
        type: ChatTypes | None = None,
        status: ChatStatuses | None = None,
        cursor: str | None = None,
    ) -> AsyncIterator[Chat]:
        while True:
            chats = await self.raw.chats.get_chats(
                user_id=self.id, type=type, status=status, cursor=cursor
            )
            if chats is None:
                return
            for c in chats.chats:
                yield self._push_chat_schema(c)
            if not chats.page_info.has_next_page:
                return
            cursor = chats.page_info.end_cursor

    async def get_chats(
        self,
        *,
        count: int = 24,
        cursor: str | None = None,
        type: ChatTypes | None = None,
        status: ChatStatuses | None = None,
    ) -> list[Chat]:
        remain = count
        resp = []
        while remain > 0:
            chats = await self.raw.chats.get_chats(
                count=min(24, remain), user_id=self.id, type=type, status=status, cursor=cursor
            )
            if chats is None:
                break
            for c in chats.chats:
                resp.append(self._push_chat_schema(c))
            if not chats.page_info.has_next_page:
                break
            cursor = chats.page_info.end_cursor
            remain -= min(24, remain)
        return resp

    async def iter_deals(
        self,
        *,
        statuses: list[ItemDealStatuses] | None = None,
        direction: ItemDealDirections | None = None,
        cursor: str | None = None,
    ) -> AsyncIterator[Deal]:
        while True:
            deals = await self.raw.deals.get_deals(
                user_id=self.id,
                statuses=statuses,
                direction=direction,
                after_cursor=cursor,
            )
            if deals is None:
                return
            for d in deals.deals:
                yield self._push_deal_schema(d)
            if not deals.page_info.has_next_page:
                return
            cursor = deals.page_info.end_cursor

    async def get_deals(
        self,
        *,
        count: int = 24,
        cursor: str | None = None,
        statuses: list[ItemDealStatuses] | None = None,
        direction: ItemDealDirections | None = None,
    ) -> list[Deal]:
        remain = count
        resp = []
        while remain > 0:
            deals = await self.raw.deals.get_deals(
                count=min(24, remain),
                user_id=self.id,
                statuses=statuses,
                direction=direction,
                after_cursor=cursor,
            )
            if deals is None:
                break
            for d in deals.deals:
                resp.append(self._push_deal_schema(d))
            if not deals.page_info.has_next_page:
                break
            cursor = deals.page_info.end_cursor
            remain -= min(24, remain)
        return resp



import os
import random

from .constants import BASE_HEADERS, EXAMPLE_USER_AGENTS


class PlayerokConfig:
    def __init__(
        self,
        *,
        user_agent: str | None = None,
        request_timeout: float | None = None,
        base_url: str | None = None,
    ):
        self.user_agent = user_agent or random.choice(EXAMPLE_USER_AGENTS)
        self.request_timeout = request_timeout or 10.0
        self.base_url = base_url or os.getenv("PLAYEROK_BASE_URL", "https://playerok.com/")

    @property
    def headers(self):
        return BASE_HEADERS | {
            "User-Agent": self.user_agent,
        }



from datetime import datetime
from typing import Any

from pydantic import Field, model_validator

from . import (
    ApiModel,
    BankCardTypes,
    PageInfo,
    TransactionDirections,
    TransactionOperations,
    TransactionPaymentMethodIds,
    TransactionProviderIds,
    TransactionStatuses,
)
from .account import UserProfile


class SBPBankMember(ApiModel):
    id: str = Field(..., alias="id")
    name: str = Field(..., alias="name")
    icon: str | None = Field(None, alias="icon")


class TransactionProviderLimitRange(ApiModel):
    min: float = Field(..., alias="min")
    max: float = Field(..., alias="max")


class TransactionProviderLimits(ApiModel):
    incoming: TransactionProviderLimitRange = Field(..., alias="incoming")
    outgoing: TransactionProviderLimitRange = Field(..., alias="outgoing")


class TransactionProviderRequiredUserData(ApiModel):
    email: bool = Field(..., alias="email")
    phone_number: bool = Field(..., alias="phoneNumber")
    erip_account_number: bool | None = Field(None, alias="eripAccountNumber")


class TransactionProviderProps(ApiModel):
    required_user_data: TransactionProviderRequiredUserData = Field(..., alias="requiredUserData")
    tooltip: str | None = Field(None, alias="tooltip")


class TransactionPaymentMethod(ApiModel):
    id: TransactionPaymentMethodIds = Field(..., alias="id")
    name: str = Field(..., alias="name")
    fee: float = Field(..., alias="fee")
    provider_id: TransactionProviderIds = Field(..., alias="providerId")
    account: Any | None = Field(None, alias="account")  # AccountProfile
    props: TransactionProviderProps = Field(..., alias="props")
    limits: TransactionProviderLimits = Field(..., alias="limits")


class TransactionProvider(ApiModel):
    id: TransactionProviderIds = Field(..., alias="id")
    name: str = Field(..., alias="name")
    fee: float = Field(..., alias="fee")
    min_fee_amount: float | None = Field(None, alias="minFeeAmount")
    description: str | None = Field(None, alias="description")
    account: Any | None = Field(None, alias="account")  # AccountProfile
    props: TransactionProviderProps = Field(..., alias="props")
    limits: TransactionProviderLimits = Field(..., alias="limits")
    payment_methods: list[TransactionPaymentMethod] | None = Field(None, alias="paymentMethods")


class Transaction(ApiModel):
    id: str = Field(..., alias="id")
    operation: TransactionOperations | None = Field(None, alias="operation")
    direction: TransactionDirections | None = Field(None, alias="direction")
    provider_id: TransactionProviderIds | None = Field(None, alias="providerId")
    provider: TransactionProvider | None = Field(None, alias="provider")
    user: UserProfile | None = Field(None, alias="user")
    creator: UserProfile | None = Field(None, alias="creator")
    status: TransactionStatuses | None = Field(None, alias="status")
    status_description: str | None = Field(None, alias="statusDescription")
    status_expiration_date: datetime | None = Field(None, alias="statusExpirationDate")
    value: float | None = Field(None, alias="value")
    fee: float | None = Field(None, alias="fee")
    created_at: datetime | None = Field(None, alias="createdAt")
    verified_at: datetime | None = Field(None, alias="verifiedAt")
    verified_by: UserProfile | None = Field(None, alias="verifiedBy")
    completed_at: datetime | None = Field(None, alias="completedAt")
    completed_by: UserProfile | None = Field(None, alias="completedBy")
    payment_method_id: str | None = Field(None, alias="paymentMethodId")
    is_suspicious: bool | None = Field(None, alias="isSuspicious")
    sbp_bank_name: str | None = Field(None, alias="spbBankName")


class TransactionList(ApiModel):
    transactions: list[Transaction] = Field(..., alias="transactions")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "transactions" not in data:
            edges = data.get("edges") or []
            data["transactions"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data


class UserBankCard(ApiModel):
    id: str = Field(..., alias="id")
    card_first_six: str = Field(..., alias="cardFirstSix")
    card_last_four: str = Field(..., alias="cardLastFour")
    card_type: BankCardTypes = Field(..., alias="cardType")
    is_chosen: bool = Field(..., alias="isChosen")


class UserBankCardList(ApiModel):
    bank_cards: list[UserBankCard] = Field(..., alias="bankCards")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "bankCards" not in data:
            edges = data.get("edges") or []
            data["bankCards"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data



from __future__ import annotations

from ..core.exceptions import Unauthorized
from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import Account, AccountProfile, UserProfile
from ..transport import PlayerokTransport


class RawAccountService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_me(self) -> Account:
        response = await self._transport.request("post", "graphql", GQL.get_me())
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "viewer"))
        if data is None:
            raise Unauthorized()
        return Account(**data)

    async def get_account(self, username: str | None = None) -> AccountProfile | None:
        if username is None:
            raise ValueError("Can't get account with no username")
        response = await self._transport.request("post", "graphql", GQL.get_user(username=username))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "user"))
        if data.get("__typename") == "User":
            profile = data
        else:
            return None
        return AccountProfile(**profile)

    async def get_user(
        self, username: str | None = None, id: str | None = None
    ) -> UserProfile | None:
        if username is None and id is None:
            raise ValueError("Can't get user with no username or id")
        response = await self._transport.request(
            "post", "graphql", GQL.get_user(username=username, id=id)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "user"))
        if data.get("__typename") == "UserFragment":
            profile = data
        elif data.get("__typename") == "User":
            profile = data.get("profile")
        else:
            return None
        return UserProfile(**profile)



from __future__ import annotations

import asyncio
import time
from collections.abc import Awaitable, Callable
from dataclasses import dataclass
from typing import Generic, TypeVar

K = TypeVar("K", bound=str)
V = TypeVar("V")


@dataclass(slots=True)
class _Entry(Generic[V]):
    value: V
    updated_at: float  # monotonic seconds


class AsyncTTLCache(Generic[K, V]):
    """
    A small async-friendly TTL cache with in-flight de-duplication.

    Notes:
    - TTL is based on monotonic time (not wall-clock).
    - `get_or_fetch()` ensures only one fetcher runs per key at a time.
    """

    def __init__(self, ttl_seconds: float):
        self._ttl = float(ttl_seconds)
        self._items: dict[K, _Entry[V]] = {}
        self._in_flight: dict[K, asyncio.Future[V]] = {}
        self._lock = asyncio.Lock()

    def _now(self) -> float:
        return time.monotonic()

    def is_fresh(self, key: K) -> bool:
        entry = self._items.get(key)
        if entry is None:
            return False
        return (self._now() - entry.updated_at) <= self._ttl

    def get(self, key: K) -> V | None:
        entry = self._items.get(key)
        if entry is None:
            return None
        if (self._now() - entry.updated_at) > self._ttl:
            return None
        return entry.value

    def set(self, key: K, value: V) -> None:
        self._items[key] = _Entry(value=value, updated_at=self._now())

    def get_or_create(self, key: K, factory: Callable[[], V]) -> V:
        """
        Identity-map helper: return existing object instance for key, otherwise create it once.

        The created entry is marked as stale until explicitly refreshed/touched.
        """
        existing = self._items.get(key)
        if existing is not None:
            return existing.value
        value = factory()
        self._items[key] = _Entry(value=value, updated_at=0.0)
        return value

    def touch(self, key: K) -> None:
        entry = self._items.get(key)
        if entry is None:
            return
        entry.updated_at = self._now()

    async def get_or_fetch(self, key: K, fetcher: Callable[[], Awaitable[V]]) -> V:
        """
        Return cached value if fresh, otherwise run `fetcher()` exactly once per key concurrently.
        """
        async with self._lock:
            cached = self.get(key)
            if cached is not None:
                return cached

            in_flight = self._in_flight.get(key)
            if in_flight is not None:
                future = in_flight
                owner = False
            else:
                loop = asyncio.get_running_loop()
                future = loop.create_future()
                self._in_flight[key] = future
                owner = True

        if owner:
            try:
                value = await fetcher()
            except Exception as e:  # noqa: BLE001
                async with self._lock:
                    if self._in_flight.get(key) is future:
                        self._in_flight.pop(key, None)
                future.set_exception(e)
                raise
            else:
                self.set(key, value)
                async with self._lock:
                    if self._in_flight.get(key) is future:
                        self._in_flight.pop(key, None)
                future.set_result(value)
                return value

        return await future



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, AsyncIterator

from ..schemas import ChatTypes
from ..schemas.chats import Chat as SchemaChat
from ..schemas.chats import ChatMessage as SchemaChatMessage

if TYPE_CHECKING:  # pragma: no cover
    from ..playerok import Playerok
    from .file import File
    from .user import User


@dataclass(slots=True)
class ChatMessage:
    id: str
    sent_at: datetime
    is_read: bool
    text: str | None = None
    file: File | None = None

    _client: "Playerok" = field(repr=False, default=None)
    _chat_id: str | None = field(repr=False, default=None)
    _user_id: str | None = field(repr=False, default=None)

    @classmethod
    def from_schema(cls, client: "Playerok", f: SchemaChatMessage, chat_id: str) -> ChatMessage:
        from .file import File

        return cls(
            id=f.id,
            sent_at=f.created_at,
            is_read=f.is_read,
            text=f.text,
            file=File.from_schema(f.file) if f.file else None,
            _client=client,
            _chat_id=chat_id,
            _user_id=f.user.id if f.user else None,
        )

    @property
    def user(self) -> "User | None":
        if not self._user_id:
            return None
        return self._client._get_user_identity(self._user_id)

    @property
    def chat(self) -> "Chat | None":
        if not self._chat_id:
            return None
        return self._client._get_chat_identity(self._chat_id)


@dataclass(slots=True)
class Chat:
    id: str
    type: ChatTypes = ChatTypes.PM
    unread_messages_counter: int | None = None

    _client: "Playerok" = field(repr=False, default=None)
    _user_id: str | None = field(repr=False, default=None)

    @classmethod
    def _create(cls, *, client: "Playerok", id: str) -> "Chat":
        return cls(id=id, _client=client)

    async def refresh(self) -> "Chat":
        await self._client._push_chat(self.id)
        return self

    def _merge_schema(self, chat: SchemaChat) -> None:
        self.unread_messages_counter = chat.unread_messages_counter
        self.type = chat.type

        # PM chat: "participants" contains both users, pick a non-self if possible.
        if chat.users:
            me_id = self._client.id
            other = None
            for u in chat.users:
                if me_id and u.id != me_id:
                    other = u
                    break
            if other is None:
                other = chat.users[0]
            self._user_id = other.id
            self._client._push_user_profile(other)

    @property
    def user(self) -> "User | None":
        if not self._user_id:
            return None
        return self._client._get_user_identity(self._user_id)

    async def send_photo(self, path: str, *, mark_as_read: bool = False) -> None:
        await self._client.raw.chats.send_message(
            self.id, photo_path=path, mark_as_read=mark_as_read
        )

    async def send_text(self, text: str, *, mark_as_read: bool = False) -> None:
        await self._client.raw.chats.send_message(self.id, text=text, mark_as_read=mark_as_read)

    async def iter_messages(
        self,
        *,
        cursor: str | None = None,
    ) -> AsyncIterator[ChatMessage]:
        while True:
            messages = await self._client.raw.chats.get_chat_messages(
                chat_id=self.id,
                after_cursor=cursor,
            )
            if messages is None:
                return
            for message in messages.messages:
                yield ChatMessage.from_schema(self._client, message, self.id)
            if not messages.page_info.has_next_page:
                break
            cursor = messages.page_info.end_cursor

    async def get_messages(
        self, *, count: int = 24, cursor: str | None = None
    ) -> list[ChatMessage]:
        remain = count
        resp = []
        while remain > 0:
            messages = await self._client.raw.chats.get_chat_messages(
                chat_id=self.id,
                after_cursor=cursor,
            )
            if messages is None:
                break
            for message in messages.messages:
                resp.append(ChatMessage.from_schema(self._client, message, self.id))
            if not messages.page_info.has_next_page:
                break
            cursor = messages.page_info.end_cursor
            remain -= min(24, remain)

        return resp



import json
from typing import Any

from .schemas import (
    ChatStatuses,
    ChatTypes,
    GameCategoryDataFieldTypes,
    GameCategoryInstructionTypes,
    GameType,
    ItemDealDirections,
    ItemDealStatuses,
    ItemsSortOptions,
    QueryID,
    SortDirections,
    TransactionOperations,
    TransactionPaymentMethodIds,
    TransactionProviderDirections,
    TransactionProviderIds,
    TransactionStatuses,
)


def _persisted(
    operation_name: str,
    variables: dict[str, Any],
    *,
    sha256_hash: Any | None = None,
    query: str | None = None,
) -> dict[str, Any]:
    r = {
        "operationName": operation_name,
        "variables": variables,
    }
    if query:
        r["query"] = query
    if sha256_hash:
        r["extensions"] = {
            "persistedQuery": {
                "version": 1,
                "sha256Hash": sha256_hash,
            }
        }
    return r


class GraphQLQuery:
    # ------------ Accounts ------------
    @staticmethod
    def get_me():
        return _persisted(
            operation_name="viewer",
            variables={},
            query="query viewer {\n  viewer {\n    ...Viewer\n    __typename\n  }\n}\n\nfragment Viewer on User {\n  id\n  username\n  email\n  role\n  hasFrozenBalance\n  supportChatId\n  systemChatId\n  unreadChatsCounter\n  isBlocked\n  isBlockedFor\n  isFundsProtectionActive\n  createdAt\n  lastItemCreatedAt\n  hasConfirmedPhoneNumber\n  canPublishItems\n  chosenVerifiedCard {\n    ...MinimalUserBankCard\n    __typename\n  }\n  profile {\n    id\n    avatarURL\n    testimonialCounter\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalUserBankCard on UserBankCard {\n  id\n  cardFirstSix\n  cardLastFour\n  cardType\n  isChosen\n  __typename\n}",
        )

    @staticmethod
    def get_user(username: str | None = None, id: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="user",
            variables={"id": id, "username": username, "hasSupportAccess": False},
            sha256_hash=QueryID.user.value,
        )

    # ------------ Games ------------

    @staticmethod
    def get_games(
        count: int = 24, type: GameType | None = None, cursor: str | None = None
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="games",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"type": type.name} if type else {},
            },
            sha256_hash=QueryID.games.value,
        )

    @staticmethod
    def get_game(id: str | None = None, slug: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="GamePage",
            variables={"id": id, "slug": slug},
            sha256_hash=QueryID.game.value,
        )

    @staticmethod
    def get_game_category(
        game_id: str | None = None,
        slug: str | None = None,
        id: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="GamePageCategory",
            variables={"id": id, "gameId": game_id, "slug": slug},
            sha256_hash=QueryID.game_category.value,
        )

    @staticmethod
    def get_game_category_agreements(
        game_category_id: str,
        user_id: str | None = None,
        count: int = 24,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryAgreements",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"gameCategoryId": game_category_id, "userId": user_id},
            },
            sha256_hash=QueryID.game_category_agreements.value,
        )

    @staticmethod
    def get_game_category_obtaining_types(
        game_category_id: str,
        count: int = 24,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryObtainingTypes",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"gameCategoryId": game_category_id},
            },
            sha256_hash=QueryID.game_category_obtaining_types.value,
        )

    @staticmethod
    def get_game_category_instructions(
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryInstructionTypes | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryInstructions",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {
                    "gameCategoryId": game_category_id,
                    "obtainingTypeId": obtaining_type_id,
                    "type": type.name if type else None,
                },
            },
            sha256_hash=QueryID.game_category_instructions.value,
        )

    @staticmethod
    def get_game_category_data_fields(
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryDataFieldTypes | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryDataFields",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {
                    "gameCategoryId": game_category_id,
                    "obtainingTypeId": obtaining_type_id,
                    "type": type.name if type else None,
                },
            },
            sha256_hash=QueryID.game_category_data_fields.value,
        )

    # ------------ Items ------------

    @staticmethod
    def get_items(
        count: int = 24,
        cursor: str | None = None,
        game_id: str | None = None,
        user_id: str | None = None,
        category_id: str | None = None,
        minimal_price: int | None = None,
        maximal_price: int | None = None,
        has_discount: bool | None = None,
        has_reviews: bool | None = None,
        attributes: list[dict[str, str]] | None = None,
        search: str | None = None,
        sort: ItemsSortOptions | None = None,
    ) -> dict[str, Any]:
        filters = {
            "gameId": game_id,
            "gameCategoryId": category_id,
            "userId": user_id,
            "status": ["APPROVED", "PENDING_MODERATION", "PENDING_APPROVAL"],
            "withOfficial": False,
        }
        sort_field = None

        if minimal_price or maximal_price:
            filters["price"] = {"min": minimal_price, "max": maximal_price}
        if has_discount:
            filters["hasDiscount"] = has_discount
        if has_reviews:
            filters["hasTestimonials"] = has_reviews
        if attributes:
            filters["attributes"] = attributes
        if search:
            filters["searchQuery"] = search
        if sort:
            if "PRICE" in sort:
                field = "price"
            else:
                field = "userRating"
            if "ASC" in sort:
                direction = "ASC"
            else:
                direction = "DESC"

            sort_field = {"field": field, "direction": direction}

        return _persisted(
            operation_name="items",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": filters,
                "sort": sort_field,
            },
            sha256_hash=QueryID.items.value,
        )

    @staticmethod
    def get_item(id: str | None = None, slug: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="item",
            variables={
                "id": id,
                "slug": slug,
                "hasSupportAccess": False,
                "showForbiddenImage": True,
            },
            sha256_hash=QueryID.item.value,
        )

    @staticmethod
    def create_item(
        game_category_id: str,
        obtaining_type_id: str,
        name: str,
        price: int,
        description: str,
        attributes: dict[str, Any],
        data_fields: list[dict[str, Any]],
        attachments_count: int,
    ) -> tuple[dict[str, Any], dict[str, Any]]:
        operations = {
            "operationName": "createItem",
            "query": "mutation createItem($input: CreateItemInput!, $attachments: [Upload!]!) {\n  createItem(input: $input, attachments: $attachments) {\n    ...RegularItem\n    __typename\n  }\n}\n\nfragment RegularItem on Item {\n  ...RegularMyItem\n  ...RegularForeignItem\n  __typename\n}\n\nfragment RegularMyItem on MyItem {\n  ...ItemFields\n  prevPrice\n  priority\n  sequence\n  priorityPrice\n  statusExpirationDate\n  comment\n  viewsCounter\n  statusDescription\n  editable\n  statusPayment {\n    ...StatusPaymentTransaction\n    __typename\n  }\n  moderator {\n    id\n    username\n    __typename\n  }\n  approvalDate\n  deletedAt\n  createdAt\n  updatedAt\n  mayBePublished\n  prevFeeMultiplier\n  sellerNotifiedAboutFeeChange\n  __typename\n}\n\nfragment ItemFields on Item {\n  id\n  slug\n  name\n  description\n  rawPrice\n  price\n  attributes\n  status\n  priorityPosition\n  sellerType\n  feeMultiplier\n  user {\n    ...ItemUser\n    __typename\n  }\n  buyer {\n    ...ItemUser\n    __typename\n  }\n  attachments {\n    ...PartialFile\n    __typename\n  }\n  category {\n    ...RegularGameCategory\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  comment\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...GameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment ItemUser on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment RegularGameCategory on GameCategory {\n  id\n  slug\n  name\n  categoryId\n  gameId\n  obtaining\n  options {\n    ...RegularGameCategoryOption\n    __typename\n  }\n  props {\n    ...GameCategoryProps\n    __typename\n  }\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  useCustomObtaining\n  autoConfirmPeriod\n  autoModerationMode\n  agreements {\n    ...RegularGameCategoryAgreement\n    __typename\n  }\n  feeMultiplier\n  __typename\n}\n\nfragment RegularGameCategoryOption on GameCategoryOption {\n  id\n  group\n  label\n  type\n  field\n  value\n  valueRangeLimit {\n    min\n    max\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryProps on GameCategoryPropsObjectType {\n  minTestimonials\n  minTestimonialsForSeller\n  __typename\n}\n\nfragment RegularGameCategoryAgreement on GameCategoryAgreement {\n  description\n  gameCategoryId\n  gameCategoryObtainingTypeId\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment StatusPaymentTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  props {\n    paymentURL\n    __typename\n  }\n  __typename\n}\n\nfragment RegularForeignItem on ForeignItem {\n  ...ItemFields\n  __typename\n}",
            "variables": {
                "input": {
                    "gameCategoryId": game_category_id,
                    "obtainingTypeId": obtaining_type_id,
                    "name": name,
                    "price": price,
                    "description": description,
                    "attributes": attributes,
                    "dataFields": data_fields,
                },
                "attachments": [None] * attachments_count,
            },
        }
        map_data = {str(i + 1): [f"variables.attachments.{i}"] for i in range(attachments_count)}
        return {
            "operations": json.dumps(operations),
            "map": json.dumps(map_data),
        }

    @staticmethod
    def get_item_priority_statuses(item_id: str, price: int) -> dict[str, Any]:
        return _persisted(
            operation_name="itemPriorityStatuses",
            variables={"itemId": item_id, "price": price},
            sha256_hash=QueryID.item_priority_statuses.value,
        )

    @staticmethod
    def increase_item_priority_status(
        item_id: str,
        priority_status_id: str,
        payment_method_id: str | None = None,
        transaction_provider_id: str = "LOCAL",
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="increaseItemPriorityStatus",
            variables={
                "input": {
                    "itemId": item_id,
                    "priorityStatuses": [priority_status_id],
                    "transactionProviderData": {
                        "paymentMethodId": payment_method_id,
                    },
                    "transactionProviderId": transaction_provider_id,
                }
            },
            query="mutation increaseItemPriorityStatus($input: PublishItemInput!) {\n  increaseItemPriorityStatus(input: $input) {\n    ...RegularItem\n    __typename\n  }\n}\n\nfragment RegularItem on Item {\n  ...RegularMyItem\n  ...RegularForeignItem\n  __typename\n}\n\nfragment RegularMyItem on MyItem {\n  ...ItemFields\n  prevPrice\n  priority\n  sequence\n  priorityPrice\n  statusExpirationDate\n  comment\n  viewsCounter\n  statusDescription\n  editable\n  statusPayment {\n    ...StatusPaymentTransaction\n    __typename\n  }\n  moderator {\n    id\n    username\n    __typename\n  }\n  approvalDate\n  deletedAt\n  createdAt\n  updatedAt\n  mayBePublished\n  prevFeeMultiplier\n  sellerNotifiedAboutFeeChange\n  __typename\n}\n\nfragment ItemFields on Item {\n  id\n  slug\n  name\n  description\n  rawPrice\n  price\n  attributes\n  status\n  priorityPosition\n  sellerType\n  feeMultiplier\n  user {\n    ...ItemUser\n    __typename\n  }\n  buyer {\n    ...ItemUser\n    __typename\n  }\n  attachments {\n    ...PartialFile\n    __typename\n  }\n  category {\n    ...RegularGameCategory\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  comment\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...GameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment ItemUser on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment RegularGameCategory on GameCategory {\n  id\n  slug\n  name\n  categoryId\n  gameId\n  obtaining\n  options {\n    ...RegularGameCategoryOption\n    __typename\n  }\n  props {\n    ...GameCategoryProps\n    __typename\n  }\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  useCustomObtaining\n  autoConfirmPeriod\n  autoModerationMode\n  agreements {\n    ...RegularGameCategoryAgreement\n    __typename\n  }\n  feeMultiplier\n  __typename\n}\n\nfragment RegularGameCategoryOption on GameCategoryOption {\n  id\n  group\n  label\n  type\n  field\n  value\n  valueRangeLimit {\n    min\n    max\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryProps on GameCategoryPropsObjectType {\n  minTestimonials\n  minTestimonialsForSeller\n  __typename\n}\n\nfragment RegularGameCategoryAgreement on GameCategoryAgreement {\n  description\n  gameCategoryId\n  gameCategoryObtainingTypeId\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment StatusPaymentTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  props {\n    paymentURL\n    __typename\n  }\n  __typename\n}\n\nfragment RegularForeignItem on ForeignItem {\n  ...ItemFields\n  __typename\n}",
        )

    @staticmethod
    def update_item(
        id: str,
        name: str | None = None,
        price: int | None = None,
        description: str | None = None,
        attributes: dict[str, Any] | None = None,
        data_fields: list[dict[str, Any]] | None = None,
        removed_attachments: list[str] | None = None,
        attachments_count: int = 0,
    ) -> dict[str, Any]:
        variables_input: dict[str, Any] = {"id": id}
        if name:
            variables_input["name"] = name
        if price is not None:
            variables_input["price"] = price
        if description:
            variables_input["description"] = description
        if attributes:
            variables_input["attributes"] = attributes
        if data_fields:
            variables_input["dataFields"] = data_fields
        if removed_attachments:
            variables_input["removedAttachments"] = removed_attachments

        operations = {
            "operationName": "updateItem",
            "query": "mutation updateItem($input: UpdateItemInput!, $addedAttachments: [Upload!]) {\n  updateItem(input: $input, addedAttachments: $addedAttachments) {\n    ...RegularItem\n    __typename\n  }\n}\n\nfragment RegularItem on Item {\n  ...RegularMyItem\n  ...RegularForeignItem\n  __typename\n}\n\nfragment RegularMyItem on MyItem {\n  ...ItemFields\n  prevPrice\n  priority\n  sequence\n  priorityPrice\n  statusExpirationDate\n  comment\n  viewsCounter\n  statusDescription\n  editable\n  statusPayment {\n    ...StatusPaymentTransaction\n    __typename\n  }\n  moderator {\n    id\n    username\n    __typename\n  }\n  approvalDate\n  deletedAt\n  createdAt\n  updatedAt\n  mayBePublished\n  prevFeeMultiplier\n  sellerNotifiedAboutFeeChange\n  __typename\n}\n\nfragment ItemFields on Item {\n  id\n  slug\n  name\n  description\n  rawPrice\n  price\n  attributes\n  status\n  priorityPosition\n  sellerType\n  feeMultiplier\n  user {\n    ...ItemUser\n    __typename\n  }\n  buyer {\n    ...ItemUser\n    __typename\n  }\n  attachments {\n    ...PartialFile\n    __typename\n  }\n  category {\n    ...RegularGameCategory\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  comment\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...GameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment ItemUser on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment RegularGameCategory on GameCategory {\n  id\n  slug\n  name\n  categoryId\n  gameId\n  obtaining\n  options {\n    ...RegularGameCategoryOption\n    __typename\n  }\n  props {\n    ...GameCategoryProps\n    __typename\n  }\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  useCustomObtaining\n  autoConfirmPeriod\n  autoModerationMode\n  agreements {\n    ...RegularGameCategoryAgreement\n    __typename\n  }\n  feeMultiplier\n  __typename\n}\n\nfragment RegularGameCategoryOption on GameCategoryOption {\n  id\n  group\n  label\n  type\n  field\n  value\n  valueRangeLimit {\n    min\n    max\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryProps on GameCategoryPropsObjectType {\n  minTestimonials\n  minTestimonialsForSeller\n  __typename\n}\n\nfragment RegularGameCategoryAgreement on GameCategoryAgreement {\n  description\n  gameCategoryId\n  gameCategoryObtainingTypeId\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment StatusPaymentTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  props {\n    paymentURL\n    __typename\n  }\n  __typename\n}\n\nfragment RegularForeignItem on ForeignItem {\n  ...ItemFields\n  __typename\n}",
            "variables": {
                "input": variables_input,
                "addedAttachments": [None] * attachments_count if attachments_count > 0 else None,
            },
        }
        map_data = (
            {str(i + 1): [f"variables.addedAttachments.{i}"] for i in range(attachments_count)}
            if attachments_count > 0
            else {}
        )

        if attachments_count > 0:
            return {
                "operations": json.dumps(operations),
                "map": json.dumps(map_data),
            }

        return operations

    @staticmethod
    def remove_item(id: str) -> dict[str, Any]:
        return {
            "operationName": "removeItem",
            "query": "mutation removeItem($id: UUID!) {\n  removeItem(id: $id) {\n    ...RegularItem\n    __typename\n  }\n}\n\nfragment RegularItem on Item {\n  ...RegularMyItem\n  ...RegularForeignItem\n  __typename\n}\n\nfragment RegularMyItem on MyItem {\n  ...ItemFields\n  prevPrice\n  priority\n  sequence\n  priorityPrice\n  statusExpirationDate\n  comment\n  viewsCounter\n  statusDescription\n  editable\n  statusPayment {\n    ...StatusPaymentTransaction\n    __typename\n  }\n  moderator {\n    id\n    username\n    __typename\n  }\n  approvalDate\n  deletedAt\n  createdAt\n  updatedAt\n  mayBePublished\n  prevFeeMultiplier\n  sellerNotifiedAboutFeeChange\n  __typename\n}\n\nfragment ItemFields on Item {\n  id\n  slug\n  name\n  description\n  rawPrice\n  price\n  attributes\n  status\n  priorityPosition\n  sellerType\n  feeMultiplier\n  user {\n    ...ItemUser\n    __typename\n  }\n  buyer {\n    ...ItemUser\n    __typename\n  }\n  attachments {\n    ...PartialFile\n    __typename\n  }\n  category {\n    ...RegularGameCategory\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  comment\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...GameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment ItemUser on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment RegularGameCategory on GameCategory {\n  id\n  slug\n  name\n  categoryId\n  gameId\n  obtaining\n  options {\n    ...RegularGameCategoryOption\n    __typename\n  }\n  props {\n    ...GameCategoryProps\n    __typename\n  }\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  useCustomObtaining\n  autoConfirmPeriod\n  autoModerationMode\n  agreements {\n    ...RegularGameCategoryAgreement\n    __typename\n  }\n  feeMultiplier\n  __typename\n}\n\nfragment RegularGameCategoryOption on GameCategoryOption {\n  id\n  group\n  label\n  type\n  field\n  value\n  valueRangeLimit {\n    min\n    max\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryProps on GameCategoryPropsObjectType {\n  minTestimonials\n  minTestimonialsForSeller\n  __typename\n}\n\nfragment RegularGameCategoryAgreement on GameCategoryAgreement {\n  description\n  gameCategoryId\n  gameCategoryObtainingTypeId\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment StatusPaymentTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  props {\n    paymentURL\n    __typename\n  }\n  __typename\n}\n\nfragment RegularForeignItem on ForeignItem {\n  ...ItemFields\n  __typename\n}",
            "variables": {"id": id},
        }

    @staticmethod
    def publish_item(
        item_id: str,
        priority_status_id: str,
        transaction_provider_id: str,
    ) -> dict[str, Any]:
        return {
            "operationName": "publishItem",
            "query": "mutation publishItem($input: PublishItemInput!) {\n  publishItem(input: $input) {\n    ...RegularItem\n    __typename\n  }\n}\n\nfragment RegularItem on Item {\n  ...RegularMyItem\n  ...RegularForeignItem\n  __typename\n}\n\nfragment RegularMyItem on MyItem {\n  ...ItemFields\n  prevPrice\n  priority\n  sequence\n  priorityPrice\n  statusExpirationDate\n  comment\n  viewsCounter\n  statusDescription\n  editable\n  statusPayment {\n    ...StatusPaymentTransaction\n    __typename\n  }\n  moderator {\n    id\n    username\n    __typename\n  }\n  approvalDate\n  deletedAt\n  createdAt\n  updatedAt\n  mayBePublished\n  prevFeeMultiplier\n  sellerNotifiedAboutFeeChange\n  __typename\n}\n\nfragment ItemFields on Item {\n  id\n  slug\n  name\n  description\n  rawPrice\n  price\n  attributes\n  status\n  priorityPosition\n  sellerType\n  feeMultiplier\n  user {\n    ...ItemUser\n    __typename\n  }\n  buyer {\n    ...ItemUser\n    __typename\n  }\n  attachments {\n    ...PartialFile\n    __typename\n  }\n  category {\n    ...RegularGameCategory\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  comment\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...GameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment ItemUser on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment RegularGameCategory on GameCategory {\n  id\n  slug\n  name\n  categoryId\n  gameId\n  obtaining\n  options {\n    ...RegularGameCategoryOption\n    __typename\n  }\n  props {\n    ...GameCategoryProps\n    __typename\n  }\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  useCustomObtaining\n  autoConfirmPeriod\n  autoModerationMode\n  agreements {\n    ...RegularGameCategoryAgreement\n    __typename\n  }\n  feeMultiplier\n  __typename\n}\n\nfragment RegularGameCategoryOption on GameCategoryOption {\n  id\n  group\n  label\n  type\n  field\n  value\n  valueRangeLimit {\n    min\n    max\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryProps on GameCategoryPropsObjectType {\n  minTestimonials\n  minTestimonialsForSeller\n  __typename\n}\n\nfragment RegularGameCategoryAgreement on GameCategoryAgreement {\n  description\n  gameCategoryId\n  gameCategoryObtainingTypeId\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment StatusPaymentTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  props {\n    paymentURL\n    __typename\n  }\n  __typename\n}\n\nfragment RegularForeignItem on ForeignItem {\n  ...ItemFields\n  __typename\n}",
            "variables": {
                "input": {
                    "itemId": item_id,
                    "priorityStatuses": [priority_status_id],
                    "transactionProviderId": transaction_provider_id,
                }
            },
        }

    # ------------ Chats ------------

    @staticmethod
    def get_chats(
        count: int = 24,
        user_id: str | None = None,
        type: ChatTypes | None = None,
        status: ChatStatuses | None = None,
        cursor: str | None = None,
    ):
        return _persisted(
            operation_name="userChats",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {
                    "userId": user_id,
                    "type": type.name if type else None,
                    "status": status.name if status else None,
                },
                "hasSupportAccess": False,
            },
            sha256_hash=QueryID.chats.value,
        )

    @staticmethod
    def get_chat(chat_id: str):
        return _persisted(
            operation_name="chat",
            variables={"id": chat_id, "hasSupportAccess": False},
            sha256_hash=QueryID.chat.value,
        )

    @staticmethod
    def mark_chat_as_read(chat_id: str):
        return _persisted(
            operation_name="markChatAsRead",
            variables={"input": {"chatId": chat_id}},
            query="mutation markChatAsRead($input: MarkChatAsReadInput!) {\n	markChatAsRead(input: $input) {\n		...RegularChat\n		__typename\n	}\n}\n\nfragment RegularChat on Chat {\n	id\n	type\n	unreadMessagesCounter\n	bookmarked\n	isTextingAllowed\n	owner {\n		...ChatParticipant\n		__typename\n	}\n	agent {\n		...ChatParticipant\n		__typename\n	}\n	participants {\n		...ChatParticipant\n		__typename\n	}\n	deals {\n		...ChatActiveItemDeal\n		__typename\n	}\n	status\n	startedAt\n	finishedAt\n	__typename\n}\n\nfragment ChatParticipant on UserFragment {\n	...RegularUserFragment\n	__typename\n}\n\nfragment RegularUserFragment on UserFragment {\n	id\n	username\n	role\n	avatarURL\n	isOnline\n	isBlocked\n	rating\n	testimonialCounter\n	createdAt\n	supportChatId\n	systemChatId\n	__typename\n}\n\nfragment ChatActiveItemDeal on ItemDealProfile {\n	id\n	direction\n	status\n	hasProblem\n	testimonial {\n		id\n		rating\n		__typename\n	}\n	item {\n		...ChatDealItemEdgeNode\n		__typename\n	}\n	user {\n		...RegularUserFragment\n		__typename\n	}\n	__typename\n}\n\nfragment ChatDealItemEdgeNode on ItemProfile {\n	...ChatDealMyItemEdgeNode\n	...ChatDealForeignItemEdgeNode\n	__typename\n}\n\nfragment ChatDealMyItemEdgeNode on MyItemProfile {\n	id\n	slug\n	priority\n	status\n	name\n	price\n	rawPrice\n	statusExpirationDate\n	sellerType\n	attachment {\n		...PartialFile\n		__typename\n	}\n	user {\n		...UserItemEdgeNode\n		__typename\n	}\n	approvalDate\n	createdAt\n	priorityPosition\n	feeMultiplier\n	__typename\n}\n\nfragment PartialFile on File {\n	id\n	url\n	__typename\n}\n\nfragment UserItemEdgeNode on UserFragment {\n	...UserEdgeNode\n	__typename\n}\n\nfragment UserEdgeNode on UserFragment {\n	...RegularUserFragment\n	__typename\n}\n\nfragment ChatDealForeignItemEdgeNode on ForeignItemProfile {\n	id\n	slug\n	priority\n	status\n	name\n	price\n	rawPrice\n	sellerType\n	attachment {\n		...PartialFile\n		__typename\n	}\n	user {\n		...UserItemEdgeNode\n		__typename\n	}\n	approvalDate\n	priorityPosition\n	createdAt\n	feeMultiplier\n	__typename\n}",
        )

    @staticmethod
    def get_chat_messages(chat_id: str, count: int = 24, after_cursor: str | None = None):
        return _persisted(
            operation_name="chatMessages",
            variables={
                "pagination": {"first": count, "after": after_cursor},
                "filter": {
                    "chatId": chat_id,
                },
                "hasSupportAccess": False,
            },
            sha256_hash=QueryID.chat_messages.value,
        )

    # ------------ Transactions ------------

    @staticmethod
    def get_transaction_providers(
        direction: TransactionProviderDirections = TransactionProviderDirections.IN,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="transactionProviders",
            variables={"filter": {"direction": direction.name if direction else None}},
            sha256_hash=QueryID.transaction_providers.value,
        )

    @staticmethod
    def get_transactions(
        user_id: str,
        count: int = 24,
        operation: list[TransactionOperations] | None = None,
        min_value: int | None = None,
        max_value: int | None = None,
        provider_id: list[TransactionProviderIds] | None = None,
        status: list[TransactionStatuses] | None = None,
        after_cursor: str | None = None,
    ) -> dict[str, Any]:
        filters: dict[str, Any] = {"userId": user_id}
        if operation:
            filters["operation"] = [op.name for op in operation]
        if min_value or max_value:
            filters["value"] = {}
            if min_value:
                filters["value"]["min"] = str(min_value)
            if max_value:
                filters["value"]["max"] = str(max_value)
        if provider_id:
            filters["providerId"] = [pid.name for pid in provider_id]
        if status:
            filters["status"] = [s.name for s in status]

        return _persisted(
            operation_name="transactions",
            variables={
                "pagination": {"first": count, "after": after_cursor},
                "filter": filters,
                "hasSupportAccess": False,
            },
            sha256_hash=QueryID.transactions.value,
        )

    @staticmethod
    def get_sbp_bank_members() -> dict[str, Any]:
        return _persisted(
            operation_name="SbpBankMembers",
            variables={},
            sha256_hash=QueryID.sbp_bank_members.value,
        )

    @staticmethod
    def get_verified_cards(
        count: int = 24,
        after_cursor: str | None = None,
        direction: SortDirections = SortDirections.ASC,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="verifiedCards",
            variables={
                "pagination": {"first": count, "after": after_cursor},
                "sort": {"direction": direction.name},
                "field": "createdAt",
            },
            sha256_hash=QueryID.verified_cards.value,
        )

    @staticmethod
    def delete_card(card_id: str) -> dict[str, Any]:
        return {
            "operationName": "deleteCard",
            "query": "mutation deleteCard($input: DeleteCardInput!) {\n  deleteCard(input: $input)\n}",
            "variables": {"input": {"cardId": card_id}},
        }

    @staticmethod
    def request_withdrawal(
        provider: TransactionProviderIds,
        account: str,
        value: int,
        payment_method_id: TransactionPaymentMethodIds | None = None,
        sbp_bank_member_id: str | None = None,
    ) -> dict[str, Any]:
        return {
            "operationName": "requestWithdrawal",
            "query": "mutation requestWithdrawal($input: CreateWithdrawalTransactionInput!) {\n  requestWithdrawal(input: $input) {\n    ...RegularTransaction\n    __typename\n  }\n}\n\nfragment RegularTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  provider {\n    ...RegularTransactionProvider\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  fee\n  createdAt\n  props {\n    ...RegularTransactionProps\n    __typename\n  }\n  verifiedAt\n  verifiedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  completedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  paymentMethodId\n  completedAt\n  isSuspicious\n  spbBankName\n  __typename\n}\n\nfragment RegularTransactionProvider on TransactionProvider {\n  id\n  name\n  fee\n  minFeeAmount\n  description\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  paymentMethods {\n    ...TransactionPaymentMethod\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProviderAccount on TransactionProviderAccount {\n  id\n  value\n  userId\n  providerId\n  paymentMethodId\n  __typename\n}\n\nfragment TransactionProviderPropsFragment on TransactionProviderPropsFragment {\n  requiredUserData {\n    ...TransactionProviderRequiredUserData\n    __typename\n  }\n  tooltip\n  __typename\n}\n\nfragment TransactionProviderRequiredUserData on TransactionProviderRequiredUserData {\n  email\n  phoneNumber\n  eripAccountNumber\n  __typename\n}\n\nfragment ProviderLimits on ProviderLimits {\n  incoming {\n    ...ProviderLimitRange\n    __typename\n  }\n  outgoing {\n    ...ProviderLimitRange\n    __typename\n  }\n  __typename\n}\n\nfragment ProviderLimitRange on ProviderLimitRange {\n  min\n  max\n  __typename\n}\n\nfragment TransactionPaymentMethod on TransactionPaymentMethod {\n  id\n  name\n  fee\n  providerId\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment RegularTransactionProps on TransactionPropsFragment {\n  creatorId\n  dealId\n  paidFromPendingIncome\n  paymentURL\n  successURL\n  fee\n  paymentAccount {\n    id\n    value\n    __typename\n  }\n  paymentGateway\n  alreadySpent\n  exchangeRate\n  amountAfterConversionRub\n  amountAfterConversionUsdt\n  userData {\n    account\n    email\n    ipAddress\n    phoneNumber\n    __typename\n  }\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}",
            "variables": {
                "input": {
                    "provider": provider.name,
                    "account": account,
                    "value": value,
                    "providerData": {
                        "paymentMethodId": payment_method_id.name if payment_method_id else None,
                        "sbpBankMemberId": sbp_bank_member_id if sbp_bank_member_id else None,
                    },
                }
            },
        }

    @staticmethod
    def remove_transaction(transaction_id: str) -> dict[str, Any]:
        return {
            "operationName": "removeTransaction",
            "query": "mutation removeTransaction($id: UUID!) {\n  removeTransaction(id: $id) {\n    ...RegularTransaction\n    __typename\n  }\n}\n\nfragment RegularTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  provider {\n    ...RegularTransactionProvider\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  fee\n  createdAt\n  props {\n    ...RegularTransactionProps\n    __typename\n  }\n  verifiedAt\n  verifiedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  completedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  paymentMethodId\n  completedAt\n  isSuspicious\n  spbBankName\n  __typename\n}\n\nfragment RegularTransactionProvider on TransactionProvider {\n  id\n  name\n  fee\n  minFeeAmount\n  description\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  paymentMethods {\n    ...TransactionPaymentMethod\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProviderAccount on TransactionProviderAccount {\n  id\n  value\n  userId\n  providerId\n  paymentMethodId\n  __typename\n}\n\nfragment TransactionProviderPropsFragment on TransactionProviderPropsFragment {\n  requiredUserData {\n    ...TransactionProviderRequiredUserData\n    __typename\n  }\n  tooltip\n  __typename\n}\n\nfragment TransactionProviderRequiredUserData on TransactionProviderRequiredUserData {\n  email\n  phoneNumber\n  eripAccountNumber\n  __typename\n}\n\nfragment ProviderLimits on ProviderLimits {\n  incoming {\n    ...ProviderLimitRange\n    __typename\n  }\n  outgoing {\n    ...ProviderLimitRange\n    __typename\n  }\n  __typename\n}\n\nfragment ProviderLimitRange on ProviderLimitRange {\n  min\n  max\n  __typename\n}\n\nfragment TransactionPaymentMethod on TransactionPaymentMethod {\n  id\n  name\n  fee\n  providerId\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment RegularTransactionProps on TransactionPropsFragment {\n  creatorId\n  dealId\n  paidFromPendingIncome\n  paymentURL\n  successURL\n  fee\n  paymentAccount {\n    id\n    value\n    __typename\n  }\n  paymentGateway\n  alreadySpent\n  exchangeRate\n  amountAfterConversionRub\n  amountAfterConversionUsdt\n  userData {\n    account\n    email\n    ipAddress\n    phoneNumber\n    __typename\n  }\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}",
            "variables": {"id": transaction_id},
        }

    @staticmethod
    def create_chat_message(chat_id: str, text: str) -> dict[str, Any]:
        return _persisted(
            operation_name="createChatMessage",
            variables={
                "input": {
                    "chatId": chat_id,
                    "text": text,
                }
            },
            query="mutation createChatMessage($input: CreateChatMessageInput!, $file: Upload) {\n  createChatMessage(input: $input, file: $file) {\n    ...RegularChatMessage\n    __typename\n  }\n}\n\nfragment RegularChatMessage on ChatMessage {\n  id\n  text\n  createdAt\n  deletedAt\n  isRead\n  isSuspicious\n  isBulkMessaging\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  file {\n    ...PartialFile\n    __typename\n  }\n  user {\n    ...ChatMessageUserFields\n    __typename\n  }\n  deal {\n    ...ChatMessageItemDeal\n    __typename\n  }\n  item {\n    ...ItemEdgeNode\n    __typename\n  }\n  transaction {\n    ...RegularTransaction\n    __typename\n  }\n  moderator {\n    ...UserEdgeNode\n    __typename\n  }\n  eventByUser {\n    ...ChatMessageUserFields\n    __typename\n  }\n  eventToUser {\n    ...ChatMessageUserFields\n    __typename\n  }\n  isAutoResponse\n  event\n  buttons {\n    ...ChatMessageButton\n    __typename\n  }\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment ChatMessageUserFields on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment ChatMessageItemDeal on ItemDeal {\n  id\n  direction\n  status\n  statusDescription\n  hasProblem\n  user {\n    ...ChatParticipant\n    __typename\n  }\n  testimonial {\n    ...ChatMessageDealTestimonial\n    __typename\n  }\n  item {\n    id\n    name\n    price\n    slug\n    rawPrice\n    sellerType\n    user {\n      ...ChatParticipant\n      __typename\n    }\n    category {\n      id\n      __typename\n    }\n    attachments {\n      ...PartialFile\n      __typename\n    }\n    comment\n    dataFields {\n      ...GameCategoryDataFieldWithValue\n      __typename\n    }\n    obtainingType {\n      ...GameCategoryObtainingType\n      __typename\n    }\n    __typename\n  }\n  obtainingFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  chat {\n    id\n    type\n    __typename\n  }\n  transaction {\n    id\n    statusExpirationDate\n    __typename\n  }\n  statusExpirationDate\n  commentFromBuyer\n  __typename\n}\n\nfragment ChatParticipant on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment ChatMessageDealTestimonial on Testimonial {\n  id\n  status\n  text\n  rating\n  createdAt\n  updatedAt\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  moderator {\n    ...RegularUserFragment\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment ItemEdgeNode on ItemProfile {\n  ...MyItemEdgeNode\n  ...ForeignItemEdgeNode\n  __typename\n}\n\nfragment MyItemEdgeNode on MyItemProfile {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  rawPrice\n  statusExpirationDate\n  sellerType\n  attachment {\n    ...PartialFile\n    __typename\n  }\n  user {\n    ...UserItemEdgeNode\n    __typename\n  }\n  approvalDate\n  createdAt\n  priorityPosition\n  viewsCounter\n  feeMultiplier\n  __typename\n}\n\nfragment UserItemEdgeNode on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment ForeignItemEdgeNode on ForeignItemProfile {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  rawPrice\n  sellerType\n  attachment {\n    ...PartialFile\n    __typename\n  }\n  user {\n    ...UserItemEdgeNode\n    __typename\n  }\n  approvalDate\n  priorityPosition\n  createdAt\n  viewsCounter\n  feeMultiplier\n  __typename\n}\n\nfragment RegularTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  provider {\n    ...RegularTransactionProvider\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  fee\n  createdAt\n  props {\n    ...RegularTransactionProps\n    __typename\n  }\n  verifiedAt\n  verifiedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  completedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  paymentMethodId\n  completedAt\n  isSuspicious\n  __typename\n}\n\nfragment RegularTransactionProvider on TransactionProvider {\n  id\n  name\n  fee\n  minFeeAmount\n  description\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  paymentMethods {\n    ...TransactionPaymentMethod\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProviderAccount on TransactionProviderAccount {\n  id\n  value\n  userId\n  __typename\n}\n\nfragment TransactionProviderPropsFragment on TransactionProviderPropsFragment {\n  requiredUserData {\n    ...TransactionProviderRequiredUserData\n    __typename\n  }\n  tooltip\n  __typename\n}\n\nfragment TransactionProviderRequiredUserData on TransactionProviderRequiredUserData {\n  email\n  phoneNumber\n  __typename\n}\n\nfragment ProviderLimits on ProviderLimits {\n  incoming {\n    ...ProviderLimitRange\n    __typename\n  }\n  outgoing {\n    ...ProviderLimitRange\n    __typename\n  }\n  __typename\n}\n\nfragment ProviderLimitRange on ProviderLimitRange {\n  min\n  max\n  __typename\n}\n\nfragment TransactionPaymentMethod on TransactionPaymentMethod {\n  id\n  name\n  fee\n  providerId\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProps on TransactionPropsFragment {\n  creatorId\n  dealId\n  paidFromPendingIncome\n  paymentURL\n  successURL\n  fee\n  paymentAccount {\n    id\n    value\n    __typename\n  }\n  paymentGateway\n  alreadySpent\n  exchangeRate\n  amountAfterConversionRub\n  amountAfterConversionUsdt\n  __typename\n}\n\nfragment ChatMessageButton on ChatMessageButton {\n  type\n  url\n  text\n  __typename\n}",
        )

    @staticmethod
    def create_chat_message_with_photo(chat_id: str, text: str | None = None) -> dict[str, Any]:
        operations = {
            "operationName": "createChatMessage",
            "variables": {
                "input": {
                    "chatId": chat_id,
                },
                "file": None,
            },
            "query": "mutation createChatMessage($input: CreateChatMessageInput!, $file: Upload, $showForbiddenImage: Boolean) {\n  createChatMessage(input: $input, file: $file) {\n    ...RegularChatMessage\n    __typename\n  }\n}\n\nfragment RegularChatMessage on ChatMessage {\n  id\n  text\n  createdAt\n  deletedAt\n  isRead\n  isSuspicious\n  isBulkMessaging\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  file {\n    ...PartialFile\n    __typename\n  }\n  user {\n    ...ChatMessageUserFields\n    __typename\n  }\n  deal {\n    ...ChatMessageItemDeal\n    __typename\n  }\n  item {\n    ...ItemEdgeNode\n    __typename\n  }\n  transaction {\n    ...RegularTransaction\n    __typename\n  }\n  moderator {\n    ...UserEdgeNode\n    __typename\n  }\n  eventByUser {\n    ...ChatMessageUserFields\n    __typename\n  }\n  eventToUser {\n    ...ChatMessageUserFields\n    __typename\n  }\n  isAutoResponse\n  event\n  buttons {\n    ...ChatMessageButton\n    __typename\n  }\n  images {\n    ...RegularFile\n    __typename\n  }\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment ChatMessageUserFields on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment ChatMessageItemDeal on ItemDeal {\n  id\n  direction\n  status\n  statusDescription\n  hasProblem\n  user {\n    ...ChatParticipant\n    __typename\n  }\n  testimonial {\n    ...ChatMessageDealTestimonial\n    __typename\n  }\n  item {\n    id\n    name\n    price\n    slug\n    rawPrice\n    sellerType\n    user {\n      ...ChatParticipant\n      __typename\n    }\n    category {\n      id\n      __typename\n    }\n    attachments(showForbiddenImage: $showForbiddenImage) {\n      ...PartialFile\n      __typename\n    }\n    isAttachmentsForbidden\n    comment\n    dataFields {\n      ...GameCategoryDataFieldWithValue\n      __typename\n    }\n    obtainingType {\n      ...GameCategoryObtainingType\n      __typename\n    }\n    __typename\n  }\n  obtainingFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  chat {\n    id\n    type\n    __typename\n  }\n  transaction {\n    id\n    statusExpirationDate\n    __typename\n  }\n  statusExpirationDate\n  commentFromBuyer\n  __typename\n}\n\nfragment ChatParticipant on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment ChatMessageDealTestimonial on Testimonial {\n  id\n  status\n  text\n  rating\n  createdAt\n  updatedAt\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  moderator {\n    ...RegularUserFragment\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment GameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  agreements {\n    ...MinimalGameCategoryAgreement\n    __typename\n  }\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalGameCategoryAgreement on GameCategoryAgreement {\n  description\n  iconType\n  id\n  sequence\n  __typename\n}\n\nfragment ItemEdgeNode on ItemProfile {\n  ...MyItemEdgeNode\n  ...ForeignItemEdgeNode\n  __typename\n}\n\nfragment MyItemEdgeNode on MyItemProfile {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  rawPrice\n  statusExpirationDate\n  sellerType\n  attachment(showForbiddenImage: $showForbiddenImage) {\n    ...PartialFile\n    __typename\n  }\n  isAttachmentsForbidden\n  user {\n    ...UserItemEdgeNode\n    __typename\n  }\n  approvalDate\n  createdAt\n  priorityPosition\n  viewsCounter\n  feeMultiplier\n  __typename\n}\n\nfragment UserItemEdgeNode on UserFragment {\n  ...UserEdgeNode\n  __typename\n}\n\nfragment ForeignItemEdgeNode on ForeignItemProfile {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  rawPrice\n  sellerType\n  attachment(showForbiddenImage: $showForbiddenImage) {\n    ...PartialFile\n    __typename\n  }\n  isAttachmentsForbidden\n  user {\n    ...UserItemEdgeNode\n    __typename\n  }\n  approvalDate\n  priorityPosition\n  createdAt\n  viewsCounter\n  feeMultiplier\n  __typename\n}\n\nfragment RegularTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  provider {\n    ...RegularTransactionProvider\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  fee\n  createdAt\n  props {\n    ...RegularTransactionProps\n    __typename\n  }\n  verifiedAt\n  verifiedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  completedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  paymentMethodId\n  completedAt\n  isSuspicious\n  spbBankName\n  __typename\n}\n\nfragment RegularTransactionProvider on TransactionProvider {\n  id\n  name\n  fee\n  minFeeAmount\n  description\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  paymentMethods {\n    ...TransactionPaymentMethod\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProviderAccount on TransactionProviderAccount {\n  id\n  value\n  userId\n  __typename\n}\n\nfragment TransactionProviderPropsFragment on TransactionProviderPropsFragment {\n  requiredUserData {\n    ...TransactionProviderRequiredUserData\n    __typename\n  }\n  tooltip\n  __typename\n}\n\nfragment TransactionProviderRequiredUserData on TransactionProviderRequiredUserData {\n  email\n  phoneNumber\n  eripAccountNumber\n  __typename\n}\n\nfragment ProviderLimits on ProviderLimits {\n  incoming {\n    ...ProviderLimitRange\n    __typename\n  }\n  outgoing {\n    ...ProviderLimitRange\n    __typename\n  }\n  __typename\n}\n\nfragment ProviderLimitRange on ProviderLimitRange {\n  min\n  max\n  __typename\n}\n\nfragment TransactionPaymentMethod on TransactionPaymentMethod {\n  id\n  name\n  fee\n  providerId\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProps on TransactionPropsFragment {\n  creatorId\n  dealId\n  paidFromPendingIncome\n  paymentURL\n  successURL\n  fee\n  paymentAccount {\n    id\n    value\n    __typename\n  }\n  paymentGateway\n  alreadySpent\n  exchangeRate\n  amountAfterConversionRub\n  amountAfterConversionUsdt\n  userData {\n    account\n    email\n    ipAddress\n    phoneNumber\n    __typename\n  }\n  __typename\n}\n\nfragment ChatMessageButton on ChatMessageButton {\n  type\n  url\n  text\n  __typename\n}\n\nfragment RegularFile on File {\n  id\n  url\n  filename\n  mime\n  __typename\n}",
        }
        if text:
            operations["variables"]["input"]["text"] = text

        map_data = {"1": ["variables.file"]}
        return {
            "operations": json.dumps(operations),
            "map": json.dumps(map_data),
        }

    # ------------ Deals ------------

    @staticmethod
    def get_deals(
        user_id: str,
        count: int = 24,
        statuses: list[ItemDealStatuses] | None = None,
        direction: ItemDealDirections | None = None,
        after_cursor: str | None = None,
    ) -> dict[str, Any]:
        str_statuses = [status.name for status in statuses] if statuses else None
        str_direction = direction.name if direction else None
        return _persisted(
            operation_name="deals",
            variables={
                "pagination": {"first": count, "after": after_cursor},
                "filter": {
                    "userId": user_id,
                    "direction": str_direction,
                    "status": str_statuses,
                },
                "showForbiddenImage": True,
            },
            sha256_hash=QueryID.deals.value,
        )

    @staticmethod
    def get_deal(deal_id: str) -> dict[str, Any]:
        return _persisted(
            operation_name="deal",
            variables={
                "id": deal_id,
                "hasSupportAccess": False,
                "showForbiddenImage": True,
            },
            sha256_hash=QueryID.deal.value,
        )

    @staticmethod
    def update_deal(deal_id: str, new_status: ItemDealStatuses) -> dict[str, Any]:
        return _persisted(
            operation_name="updateDeal",
            variables={
                "input": {
                    "id": deal_id,
                    "status": new_status.name,
                }
            },
            query="mutation updateDeal($input: UpdateItemDealInput!) {\n  updateDeal(input: $input) {\n    ...RegularItemDeal\n    __typename\n  }\n}\n\nfragment RegularItemDeal on ItemDeal {\n  id\n  status\n  direction\n  statusExpirationDate\n  statusDescription\n  obtaining\n  hasProblem\n  reportProblemEnabled\n  completedBy {\n    ...MinimalUserFragment\n    __typename\n  }\n  props {\n    ...ItemDealProps\n    __typename\n  }\n  prevStatus\n  completedAt\n  createdAt\n  logs {\n    ...ItemLog\n    __typename\n  }\n  transaction {\n    ...ItemDealTransaction\n    __typename\n  }\n  user {\n    ...UserEdgeNode\n    __typename\n  }\n  chat {\n    ...RegularChatId\n    __typename\n  }\n  item {\n    ...PartialDealItem\n    __typename\n  }\n  testimonial {\n    ...RegularItemDealTestimonial\n    __typename\n  }\n  obtainingFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  commentFromBuyer\n  __typename\n}\n\nfragment MinimalUserFragment on UserFragment {\n  id\n  username\n  role\n  __typename\n}\n\nfragment ItemDealProps on ItemDealProps {\n  autoConfirmPeriod\n  __typename\n}\n\nfragment ItemLog on ItemLog {\n  id\n  event\n  createdAt\n  user {\n    ...UserEdgeNode\n    __typename\n  }\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment ItemDealTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  status\n  value\n  createdAt\n  paymentMethodId\n  statusExpirationDate\n  __typename\n}\n\nfragment RegularChatId on Chat {\n  id\n  __typename\n}\n\nfragment PartialDealItem on Item {\n  ...PartialDealMyItem\n  ...PartialDealForeignItem\n  __typename\n}\n\nfragment PartialDealMyItem on MyItem {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  priorityPrice\n  rawPrice\n  statusExpirationDate\n  sellerType\n  approvalDate\n  createdAt\n  priorityPosition\n  viewsCounter\n  feeMultiplier\n  comment\n  attachments {\n    ...RegularFile\n    __typename\n  }\n  user {\n    ...UserEdgeNode\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  category {\n    ...MinimalGameCategory\n    __typename\n  }\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...MinimalGameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment RegularFile on File {\n  id\n  url\n  filename\n  mime\n  __typename\n}\n\nfragment RegularGameProfile on GameProfile {\n  id\n  name\n  type\n  slug\n  logo {\n    ...PartialFile\n    __typename\n  }\n  __typename\n}\n\nfragment PartialFile on File {\n  id\n  url\n  __typename\n}\n\nfragment MinimalGameCategory on GameCategory {\n  id\n  slug\n  name\n  __typename\n}\n\nfragment GameCategoryDataFieldWithValue on GameCategoryDataFieldWithValue {\n  id\n  label\n  type\n  inputType\n  copyable\n  hidden\n  required\n  value\n  __typename\n}\n\nfragment MinimalGameCategoryObtainingType on GameCategoryObtainingType {\n  id\n  name\n  description\n  gameCategoryId\n  noCommentFromBuyer\n  instructionForBuyer\n  instructionForSeller\n  sequence\n  feeMultiplier\n  props {\n    minTestimonialsForSeller\n    __typename\n  }\n  __typename\n}\n\nfragment PartialDealForeignItem on ForeignItem {\n  id\n  slug\n  priority\n  status\n  name\n  price\n  rawPrice\n  sellerType\n  approvalDate\n  priorityPosition\n  createdAt\n  viewsCounter\n  feeMultiplier\n  comment\n  attachments {\n    ...RegularFile\n    __typename\n  }\n  user {\n    ...UserEdgeNode\n    __typename\n  }\n  game {\n    ...RegularGameProfile\n    __typename\n  }\n  category {\n    ...MinimalGameCategory\n    __typename\n  }\n  dataFields {\n    ...GameCategoryDataFieldWithValue\n    __typename\n  }\n  obtainingType {\n    ...MinimalGameCategoryObtainingType\n    __typename\n  }\n  __typename\n}\n\nfragment RegularItemDealTestimonial on Testimonial {\n  id\n  status\n  text\n  rating\n  createdAt\n  updatedAt\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  moderator {\n    ...RegularUserFragment\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  __typename\n}",
        )

    @staticmethod
    def create_deal(
        item_id: str,
        transaction_provider_id: str,
        obtaining_fields: list[dict[str, str]] | None = None,
        comment_from_buyer: str | None = None,
        payment_method_id: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="createDeal",
            variables={
                "input": {
                    "itemId": item_id,
                    "transactionProviderId": transaction_provider_id,
                    "obtainingFields": obtaining_fields or [],
                    "commentFromBuyer": comment_from_buyer,
                    "transactionProviderData": {"paymentMethodId": payment_method_id},
                }
            },
            query="mutation createDeal($input: CreateItemDealInput!) {\n  createDeal(input: $input) {\n    ...RegularTransaction\n    __typename\n  }\n}\n\nfragment RegularTransaction on Transaction {\n  id\n  operation\n  direction\n  providerId\n  provider {\n    ...RegularTransactionProvider\n    __typename\n  }\n  user {\n    ...RegularUserFragment\n    __typename\n  }\n  creator {\n    ...RegularUserFragment\n    __typename\n  }\n  status\n  statusDescription\n  statusExpirationDate\n  value\n  fee\n  createdAt\n  props {\n    ...RegularTransactionProps\n    __typename\n  }\n  verifiedAt\n  verifiedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  completedBy {\n    ...UserEdgeNode\n    __typename\n  }\n  paymentMethodId\n  completedAt\n  isSuspicious\n  spbBankName\n  __typename\n}\n\nfragment RegularTransactionProvider on TransactionProvider {\n  id\n  name\n  fee\n  minFeeAmount\n  description\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  paymentMethods {\n    ...TransactionPaymentMethod\n    __typename\n  }\n  __typename\n}\n\nfragment RegularTransactionProviderAccount on TransactionProviderAccount {\n  id\n  value\n  userId\n  providerId\n  paymentMethodId\n  __typename\n}\n\nfragment TransactionProviderPropsFragment on TransactionProviderPropsFragment {\n  requiredUserData {\n    ...TransactionProviderRequiredUserData\n    __typename\n  }\n  tooltip\n  __typename\n}\n\nfragment TransactionProviderRequiredUserData on TransactionProviderRequiredUserData {\n  email\n  phoneNumber\n  eripAccountNumber\n  __typename\n}\n\nfragment ProviderLimits on ProviderLimits {\n  incoming {\n    ...ProviderLimitRange\n    __typename\n  }\n  outgoing {\n    ...ProviderLimitRange\n    __typename\n  }\n  __typename\n}\n\nfragment ProviderLimitRange on ProviderLimitRange {\n  min\n  max\n  __typename\n}\n\nfragment TransactionPaymentMethod on TransactionPaymentMethod {\n  id\n  name\n  fee\n  providerId\n  account {\n    ...RegularTransactionProviderAccount\n    __typename\n  }\n  props {\n    ...TransactionProviderPropsFragment\n    __typename\n  }\n  limits {\n    ...ProviderLimits\n    __typename\n  }\n  __typename\n}\n\nfragment RegularUserFragment on UserFragment {\n  id\n  username\n  role\n  avatarURL\n  isOnline\n  isBlocked\n  rating\n  testimonialCounter\n  createdAt\n  supportChatId\n  systemChatId\n  __typename\n}\n\nfragment RegularTransactionProps on TransactionPropsFragment {\n  creatorId\n  dealId\n  paidFromPendingIncome\n  paymentURL\n  successURL\n  fee\n  paymentAccount {\n    id\n    value\n    __typename\n  }\n  paymentGateway\n  alreadySpent\n  exchangeRate\n  amountAfterConversionRub\n  amountAfterConversionUsdt\n  userData {\n    account\n    email\n    ipAddress\n    phoneNumber\n    __typename\n  }\n  __typename\n}\n\nfragment UserEdgeNode on UserFragment {\n  ...RegularUserFragment\n  __typename\n}",
        )



from __future__ import annotations

import typing
from datetime import datetime
from typing import Any

from pydantic import Field, model_validator

from . import (
    ApiModel,
    ItemDealDirections,
    ItemDealStatuses,
    PageInfo,
    ReviewStatuses,
)
from .account import UserProfile
from .games import GameCategoryDataField
from .items import Item, ItemLog

if typing.TYPE_CHECKING:
    from . import Chat, Moderator, Transaction


class ItemDeal(ApiModel):
    id: str = Field(..., alias="id")
    status: ItemDealStatuses = Field(..., alias="status")
    status_expiration_date: datetime | None = Field(None, alias="statusExpirationDate")
    status_description: str | None = Field(None, alias="statusDescription")
    direction: ItemDealDirections = Field(..., alias="direction")
    obtaining: str | None = Field(None, alias="obtaining")
    has_problem: bool = Field(False, alias="hasProblem")
    report_problem_enabled: bool | None = Field(None, alias="reportProblemEnabled")
    completed_user: UserProfile | None = Field(None, alias="completedBy")
    props: dict[str, Any] | None = Field(None, alias="props")
    previous_status: ItemDealStatuses | None = Field(None, alias="prevStatus")
    completed_at: datetime | None = Field(None, alias="completedAt")
    created_at: datetime | None = Field(None, alias="createdAt")
    logs: list[ItemLog] | None = Field(None, alias="logs")
    transaction: Transaction | None = Field(None, alias="transaction")
    user: UserProfile | None = Field(None, alias="user")
    chat: Chat | None = Field(None, alias="chat")
    item: Item | None = Field(None, alias="item")
    review: Review | None = Field(None, alias="testimonial")
    obtaining_fields: list[GameCategoryDataField] | None = Field(None, alias="obtainingFields")
    comment_from_buyer: str | None = Field(None, alias="commentFromBuyer")


class ItemDealList(ApiModel):
    deals: list[ItemDeal] = Field(..., alias="deals")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "deals" not in data:
            edges = data.get("edges") or []
            data["deals"] = [(edge or {}).get("node") for edge in edges if (edge or {}).get("node")]
        return data


class Review(ApiModel):
    id: str = Field(..., alias="id")
    status: ReviewStatuses = Field(..., alias="status")
    text: str | None = Field(None, alias="text")
    rating: int = Field(..., alias="rating")
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    deal: ItemDeal | None = Field(None, alias="deal")
    creator: UserProfile | None = Field(None, alias="creator")
    moderator: Moderator | None = Field(None, alias="moderator")
    user: UserProfile | None = Field(None, alias="user")


class ReviewList(ApiModel):
    reviews: list[Review] = Field(..., alias="reviews")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "reviews" not in data:
            edges = data.get("edges") or []
            data["reviews"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data


# Resolve forward references after all models are defined
from .chats import Chat, Moderator  # noqa: E402
from .transactions import Transaction  # noqa: E402

ItemDeal.model_rebuild()
ItemDealList.model_rebuild()
Review.model_rebuild()
ReviewList.model_rebuild()



from ..transport import PlayerokTransport
from .account import RawAccountService
from .chats import RawChatService
from .deals import RawDealsService
from .games import RawGamesService
from .items import RawItemsService
from .transactions import RawTransactionService


class RawAPI:
    """
    Low-level API access (raw GraphQL + parsing into Schema models).

    This layer has:
    - no caching
    - no domain methods
    - no cross-service orchestration
    """

    def __init__(self, transport: PlayerokTransport):
        self.account = RawAccountService(transport)
        self.chats = RawChatService(transport)
        self.deals = RawDealsService(transport)
        self.games = RawGamesService(transport)
        self.items = RawItemsService(transport)
        self.transactions = RawTransactionService(transport)


__all__ = [
    "RawAPI",
    "RawAccountService",
    "RawChatService",
    "RawDealsService",
    "RawGamesService",
    "RawItemsService",
    "RawTransactionService",
]



from collections.abc import Iterable
from typing import Any

from .exceptions import GraphQLError


def _dig(obj: dict[str, Any], path: Iterable[str]) -> Any:
    cur: Any = obj
    for key in path:
        if cur is None:
            return None
        cur = cur.get(key)
    return cur


def _raise_on_gql_errors(payload: dict[str, Any]) -> None:
    errors = payload.get("errors")
    if errors:
        if len(errors) >= 1:
            if "message" in errors[0]:
                raise GraphQLError(errors[0]["message"] + ": " + str(errors[0]))
            raise GraphQLError(errors[0])
        raise GraphQLError(errors)



from __future__ import annotations

from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import (
    SBPBankMember,
    SortDirections,
    Transaction,
    TransactionList,
    TransactionOperations,
    TransactionPaymentMethodIds,
    TransactionProvider,
    TransactionProviderDirections,
    TransactionProviderIds,
    TransactionStatuses,
    UserBankCardList,
)
from ..transport import PlayerokTransport


class RawTransactionService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_transaction_providers(
        self, direction: TransactionProviderDirections = TransactionProviderDirections.IN
    ) -> list[TransactionProvider]:
        response = await self._transport.request(
            "post", "graphql", GQL.get_transaction_providers(direction)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "transactionProviders")) or []
        return [TransactionProvider(**provider) for provider in data]

    async def get_transactions(
        self,
        user_id: str,
        count: int = 24,
        operation: list[TransactionOperations] | None = None,
        min_value: int | None = None,
        max_value: int | None = None,
        provider_id: list[TransactionProviderIds] | None = None,
        status: list[TransactionStatuses] | None = None,
        after_cursor: str | None = None,
    ) -> TransactionList:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_transactions(
                user_id=user_id,
                count=count,
                operation=operation,
                min_value=min_value,
                max_value=max_value,
                provider_id=provider_id,
                status=status,
                after_cursor=after_cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "transactions"))
        return TransactionList(**data)

    async def get_sbp_bank_members(self) -> list[SBPBankMember]:
        response = await self._transport.request("post", "graphql", GQL.get_sbp_bank_members())
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "sbpBankMembers")) or []
        return [SBPBankMember(**member) for member in data]

    async def get_verified_cards(
        self,
        count: int = 24,
        after_cursor: str | None = None,
        direction: SortDirections = SortDirections.ASC,
    ) -> UserBankCardList:
        response = await self._transport.request(
            "post", "graphql", GQL.get_verified_cards(count, after_cursor, direction)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "verifiedCards"))
        return UserBankCardList(**data)

    async def delete_card(self, card_id: str) -> bool:
        response = await self._transport.request("post", "graphql", GQL.delete_card(card_id))
        raw = response.json()
        _raise_on_gql_errors(raw)

        return _dig(raw, ("data", "deleteCard"))

    async def request_withdrawal(
        self,
        provider: TransactionProviderIds,
        account: str,
        value: int,
        payment_method_id: TransactionPaymentMethodIds | None = None,
        sbp_bank_member_id: str | None = None,
    ) -> Transaction:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.request_withdrawal(provider, account, value, payment_method_id, sbp_bank_member_id),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "requestWithdrawal"))
        return Transaction(**data)

    async def remove_transaction(self, transaction_id: str) -> Transaction:
        response = await self._transport.request(
            "post", "graphql", GQL.remove_transaction(transaction_id)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "removeTransaction"))
        return Transaction(**data)
